@@ -9,6 +9,8 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "depends.h"
+#include "intl.h"
 #include "misc.h"
 #include "rpmlib.h"
 #include "messages.h"
@@ -71,7 +73,6 @@ static int checkPackageDeps(rpmDependencies rpmdep, struct problemsSet * psp,
 			Header h, const char * requirement);
 static int dbrecMatchesDepFlags(rpmDependencies rpmdep, int recOffset, 
 			        char * reqVersion, int reqFlags);
-static int headerMatchesDepFlags(Header h, char * reqVersion, int reqFlags);
 struct availablePackage * alSatisfiesDepend(struct availableList * al, 
 					    char * reqName, char * reqVersion, 
 					    int reqFlags);
@@ -92,7 +93,7 @@ static void alCreate(struct availableList * al) {
 
     al->index.index = NULL;
     alFreeIndex(al);
-};
+}
 
 static void alFreeIndex(struct availableList * al) {
     if (al->index.size) {
@@ -188,7 +189,7 @@ static void alMakeIndex(struct availableList * al) {
 	    }
 	}
 
-	qsort(ai->index, ai->size, sizeof(*ai->index), (void *) indexcmp);
+	qsort(ai->index, ai->size, sizeof(*ai->index), indexcmp);
     }
 }
 
@@ -201,7 +202,7 @@ struct availablePackage * alSatisfiesDepend(struct availableList * al,
 
     needle.entry = reqName;
     match = bsearch(&needle, al->index.index, al->index.size,
-		    sizeof(*al->index.index), (void *) indexcmp);
+		    sizeof(*al->index.index), indexcmp);
  
     if (!match) return NULL;
     if (match->type != IET_NAME) return match->package;
@@ -212,7 +213,7 @@ struct availablePackage * alSatisfiesDepend(struct availableList * al,
     return NULL;
 }
 
-int indexcmp(const void * a, const void *b) {
+static int indexcmp(const void * a, const void *b) {
     const struct availableIndexEntry * aptr = a;
     const struct availableIndexEntry * bptr = b;
 
@@ -315,6 +316,7 @@ void rpmdepFreeConflicts(struct rpmDependencyConflict * conflicts, int
     int i;
 
     for (i = 0; i < numConflicts; i++) {
+	headerFree(conflicts[i].byHeader);
 	free(conflicts[i].byName);
 	free(conflicts[i].byVersion);
 	free(conflicts[i].byRelease);
@@ -378,8 +380,8 @@ int rpmdepCheck(rpmDependencies rpmdep,
 	h = rpmdbGetRecord(rpmdep->db, rpmdep->removedPackages[i]);
 	if (!h) {
 	    rpmError(RPMERR_DBCORRUPT, 
-			"cannot read header at %d for dependency "
-		  "check", rpmdep->removedPackages[i]);
+			_("cannot read header at %d for dependency check"),
+		        rpmdep->removedPackages[i]);
 	    free(ps.problems);
 	    return 1;
 	}
@@ -532,8 +534,9 @@ static int checkPackageSet(rpmDependencies rpmdep, struct problemsSet * psp,
 
 	h = rpmdbGetRecord(rpmdep->db, matches->recs[i].recOffset);
 	if (!h) {
-	    rpmError(RPMERR_DBCORRUPT, "cannot read header at %d for dependency "
-		  "check", rpmdep->removedPackages[i]);
+	    rpmError(RPMERR_DBCORRUPT, 
+                     _("cannot read header at %d for dependency check"),
+		     rpmdep->removedPackages[i]);
 	    return 1;
 	}
 
@@ -633,6 +636,7 @@ static int checkPackageDeps(rpmDependencies rpmdep, struct problemsSet * psp,
 		psp->problems = realloc(psp->problems, sizeof(*psp->problems) * 
 			    psp->alloced);
 	    }
+	    psp->problems[psp->num].byHeader = headerCopy(h);
 	    psp->problems[psp->num].byName = strdup(name);
 	    psp->problems[psp->num].byVersion = strdup(version);
 	    psp->problems[psp->num].byRelease = strdup(release);
@@ -675,6 +679,7 @@ static int checkPackageDeps(rpmDependencies rpmdep, struct problemsSet * psp,
 		psp->problems = realloc(psp->problems, sizeof(*psp->problems) * 
 			    psp->alloced);
 	    }
+	    psp->problems[psp->num].byHeader = headerCopy(h);
 	    psp->problems[psp->num].byName = strdup(name);
 	    psp->problems[psp->num].byVersion = strdup(version);
 	    psp->problems[psp->num].byRelease = strdup(release);
@@ -704,7 +709,7 @@ static int checkPackageDeps(rpmDependencies rpmdep, struct problemsSet * psp,
     return ourrc;
 }
 
-static int headerMatchesDepFlags(Header h, char * reqInfo, int reqFlags) {
+int headerMatchesDepFlags(Header h, char * reqInfo, int reqFlags) {
     char * name, * version, * release, * chptr;
     char * reqVersion = reqInfo;
     char * reqRelease = NULL;
@@ -716,7 +721,7 @@ static int headerMatchesDepFlags(Header h, char * reqInfo, int reqFlags) {
 
     headerGetEntry(h, RPMTAG_NAME, &type, (void *) &name, &count);
 
-    if (!(reqFlags & RPMSENSE_SENSEMASK)) {
+    if (!(reqFlags & RPMSENSE_SENSEMASK) || !reqInfo || !strlen(reqInfo)) {
 	return 1;
     }
 
@@ -817,7 +822,7 @@ static int addOrderedPack(rpmDependencies rpmdep,
 	    strcat(errorString, " ");
 	}
 	
-	rpmError(RPMMESS_PREREQLOOP, "loop in prerequisite chain: %s",
+	rpmError(RPMMESS_PREREQLOOP, _("loop in prerequisite chain: %s"),
 		 errorString);
 
 	return 1;
@@ -857,8 +862,14 @@ static int addOrderedPack(rpmDependencies rpmdep,
 		    rc = addOrderedPack(rpmdep, match, ordering, orderNumPtr,
 				        selected, selectionClass, 1,
 				        errorStack);
+
+		/* FIXME: I suspect we should free things here */
+		if (rc) return 1;
 	    }
 	}
+
+	free(requires);
+	free(requiresVersion);
     }
 
     /* whew -- add this package */
