@@ -10,6 +10,8 @@
 #include "config.h"
 #include "miscfn.h"
 
+#include <zlib.h>
+
 #if HAVE_ALLOCA_H
 # include <alloca.h>
 #endif
@@ -24,6 +26,7 @@
 #include <netinet/in.h>
 
 #include "header.h"
+#include "intl.h"
 #include "tread.h"
 
 #define INDEX_MALLOC_SIZE 8
@@ -328,6 +331,76 @@ Header headerRead(int fd, int magicp)
     return h;
 }
 
+void headerGzWrite(gzFile fd, Header h, int magicp)
+{
+    void * p;
+    int length;
+    int_32 l;
+
+    p = doHeaderUnload(h, &length);
+
+    if (magicp) {
+	gzwrite(fd, header_magic, sizeof(header_magic));
+	l = htonl(0);
+	gzwrite(fd, &l, sizeof(l));
+    }
+    
+    gzwrite(fd, p, length);
+
+    free(p);
+}
+
+Header headerGzRead(gzFile fd, int magicp)
+{
+    int_32 reserved;
+    int_32 * p;
+    int_32 il, dl;
+    int_32 magic;
+    Header h;
+    void * block;
+    int totalSize;
+
+    if (magicp == HEADER_MAGIC_YES) {
+	if (gzread(fd, &magic, sizeof(magic)) != sizeof(magic))
+	    return NULL;
+	if (memcmp(&magic, header_magic, sizeof(magic))) {
+	    return NULL;
+	}
+
+	if (gzread(fd, &reserved, sizeof(reserved)) != sizeof(reserved))
+	    return NULL;
+    }
+    
+    /* First read the index length (count of index entries) */
+    if (gzread(fd, &il, sizeof(il)) != sizeof(il)) 
+	return NULL;
+
+    il = ntohl(il);
+
+    /* Then read the data length (number of bytes) */
+    if (gzread(fd, &dl, sizeof(dl)) != sizeof(dl)) 
+	return NULL;
+
+    dl = ntohl(dl);
+
+    totalSize = sizeof(int_32) + sizeof(int_32) + 
+		(il * sizeof(struct entryInfo)) + dl;
+
+    block = p = malloc(totalSize);
+    *p++ = htonl(il);
+    *p++ = htonl(dl);
+
+    totalSize -= sizeof(int_32) + sizeof(int_32);
+    if (gzread(fd, p, totalSize) != totalSize)
+	return NULL;
+    
+    h = headerLoad(block);
+
+    free(block);
+
+    return h;
+}
+
 /********************************************************************/
 /*                                                                  */
 /* Header loading and unloading                                     */
@@ -725,7 +798,7 @@ int headerGetEntry(Header h, int_32 tag, int_32 * type, void **p, int_32 * c)
 	if (h->langNum == -1) headerResetLang(h);
 
 	if (type) *type = RPM_STRING_TYPE;
-	if (c) *c = entry->info.count;
+	if (c) *c = 1;
 
 	chptr = entry->data;
 	for (i = 0; i < h->langNum; i++) 
@@ -945,7 +1018,7 @@ int headerAddI18NString(Header h, int_32 tag, char * string, char * lang) {
     }
 
     if (!table && !entry) {
-	if (!lang) {
+	if (!lang || (lang[0] == 'C' && lang[1] == '\0')) {
 	    charArray[0] = "C";
 	    if (!headerAddEntry(h, HEADER_I18NTABLE, RPM_STRING_ARRAY_TYPE, 
 				&charArray, 1)) return 0;
@@ -975,7 +1048,7 @@ int headerAddI18NString(Header h, int_32 tag, char * string, char * lang) {
     }
 
     if (!entry) {
-	strArray = alloca(sizeof(*strArray) * langNum);
+	strArray = alloca(sizeof(*strArray) * (langNum + 1));
 	for (i = 0; i < langNum; i++)
 	    strArray[i] = "";
 	strArray[langNum] = string;
@@ -1004,7 +1077,7 @@ int headerAddI18NString(Header h, int_32 tag, char * string, char * lang) {
 	memcpy(buf, entry->data, length);
 	strcpy(buf + length, string);
 	i = strlen(chptr + length) + 1;
-	memcpy(buf + strlen(string) + 1, entry->data + length + i, 
+	memcpy(buf + strlen(string) + 1, ((char *) entry->data) + length + i, 
 		entry->length - length - i);
 
 	free(entry->data);
@@ -1015,6 +1088,7 @@ int headerAddI18NString(Header h, int_32 tag, char * string, char * lang) {
     return 0;
 }
 
+/* if there are multiple entries with this tag, the first one gets replaced */
 int headerModifyEntry(Header h, int_32 tag, int_32 type, void *p, int_32 c)
 {
     struct indexEntry *entry;
@@ -1022,10 +1096,14 @@ int headerModifyEntry(Header h, int_32 tag, int_32 type, void *p, int_32 c)
 
     /* First find the tag */
     entry = findEntry(h, tag, type);
-    if (! entry) {
+    if (!entry) {
 	return 0;
     }
 
+    /* make sure entry points to the first occurence of this tag */
+    while (entry > h->index && (entry - 1)->info.tag == tag)  
+	entry--;
+
     /* free after we've grabbed the new data in case the two are intertwined;
        that's a bad idea but at least we won't break */
     oldData = entry->data;
@@ -1067,9 +1145,7 @@ int headerAppendEntry(Header h, int_32 tag, int_32 type, void * p, int_32 c) {
     length = dataLength(type, p, c, 0);
 
     entry->data = realloc(entry->data, entry->length + length);
-    memcpy(((char *) entry->data) + entry->length, p, length);
-
-    copyData(type, entry->data + entry->length, p, c, length);
+    copyData(type, ((char *) entry->data) + entry->length, p, c, length);
 
     entry->length += length;
 
@@ -1099,6 +1175,7 @@ int headerRemoveEntry(Header h, int_32 tag) {
 
     last = h->index + h->indexUsed;
     while (entry->info.tag == tag && entry < last) {
+	free(entry->data);
 	*(entry++) = *(--last);
     }
     h->indexUsed = last - h->index;
@@ -1196,14 +1273,14 @@ static int parseExpression(struct sprintfToken * token, char * str,
     while (*chptr && *chptr != '?') chptr++;
 
     if (*chptr != '?') {
-	*error = "? expected in expression";
+	*error = _("? expected in expression");
 	return 1;
     }
 
     *chptr++ = '\0';;
 
     if (*chptr != '{') {
-	*error = "{ exected after ? in expression";
+	*error = _("{ exected after ? in expression");
 	return 1;
     }
 
@@ -1213,14 +1290,14 @@ static int parseExpression(struct sprintfToken * token, char * str,
 		    &token->u.cond.numIfTokens, &end, PARSER_IN_EXPR, error)) 
 	return 1;
     if (!*end) {
-	*error = "} expected in expression";
+	*error = _("} expected in expression");
 	freeFormat(token->u.cond.ifFormat, token->u.cond.numIfTokens);
 	return 1;
     }
 
     chptr = end;
     if (*chptr != ':' && *chptr != '|') {
-	*error = ": expected following ? subexpression";
+	*error = _(": expected following ? subexpression");
 	freeFormat(token->u.cond.ifFormat, token->u.cond.numIfTokens);
 	return 1;
     }
@@ -1233,7 +1310,7 @@ static int parseExpression(struct sprintfToken * token, char * str,
 	chptr++;
 
 	if (*chptr != '{') {
-	    *error = "{ exected after : in expression";
+	    *error = _("{ exected after : in expression");
 	    freeFormat(token->u.cond.ifFormat, token->u.cond.numIfTokens);
 	    return 1;
 	}
@@ -1245,14 +1322,14 @@ static int parseExpression(struct sprintfToken * token, char * str,
 			error)) 
 	    return 1;
 	if (!*end) {
-	    *error = "} expected in expression";
+	    *error = _("} expected in expression");
 	    freeFormat(token->u.cond.ifFormat, token->u.cond.numIfTokens);
 	    return 1;
 	}
 
 	chptr = end;
 	if (*chptr != '|') {
-	    *error = "| expected at end of expression";
+	    *error = _("| expected at end of expression");
 	    freeFormat(token->u.cond.ifFormat, token->u.cond.numIfTokens);
 	    freeFormat(token->u.cond.elseFormat, token->u.cond.numElseTokens);
 	    return 1;
@@ -1346,7 +1423,7 @@ static int parseFormat(char * str, const struct headerTagTableEntry * tags,
 		chptr = start;
 		while (*chptr && *chptr != '{') chptr++;
 		if (!*chptr) {
-		    *error = "missing { after %";
+		    *error = _("missing { after %");
 		    freeFormat(format, numTokens);
 		    return 1;
 		}
@@ -1374,7 +1451,7 @@ static int parseFormat(char * str, const struct headerTagTableEntry * tags,
 		next = start;
 		while (*next && *next != '}') next++;
 		if (!*next) {
-		    *error = "missing } after %{";
+		    *error = _("missing } after %{");
 		    freeFormat(format, numTokens);
 		    return 1;
 		}
@@ -1386,7 +1463,7 @@ static int parseFormat(char * str, const struct headerTagTableEntry * tags,
 		if (*chptr) {
 		    *chptr++ = '\0';
 		    if (!*chptr) {
-			*error = "empty tag format";
+			*error = _("empty tag format");
 			freeFormat(format, numTokens);
 			return 1;
 		    }
@@ -1396,11 +1473,12 @@ static int parseFormat(char * str, const struct headerTagTableEntry * tags,
 		}
 	    
 		if (!*start) {
-		    *error = "empty tag name";
+		    *error = _("empty tag name");
 		    freeFormat(format, numTokens);
 		    return 1;
 		}
 
+		i = 0;
 		findTag(start, tags, extensions, &entry, &ext);
 
 		if (entry) {
@@ -1410,7 +1488,7 @@ static int parseFormat(char * str, const struct headerTagTableEntry * tags,
 		    format[currToken].u.tag.ext = ext->u.tagFunction;
 		    format[currToken].u.tag.extNum = ext - extensions;
 		} else {
-		    *error = "unknown tag";
+		    *error = _("unknown tag");
 		    freeFormat(format, numTokens);
 		    return 1;
 		}
@@ -1436,7 +1514,7 @@ static int parseFormat(char * str, const struct headerTagTableEntry * tags,
 	    }
 
 	    if (!start) {
-		*error = "] expected at end of array";
+		*error = _("] expected at end of array");
 		freeFormat(format, numTokens);
 		return 1;
 	    }
@@ -1452,9 +1530,9 @@ static int parseFormat(char * str, const struct headerTagTableEntry * tags,
 	    if ((*start == ']' && state != PARSER_IN_ARRAY) ||
 	        (*start == '}' && state != PARSER_IN_EXPR)) {
 		if (*start == ']')
-		    *error = "unexpected ]";
+		    *error = _("unexpected ]");
 		else
-		    *error = "unexpected }";
+		    *error = _("unexpected }");
 		freeFormat(format, numTokens);
 		return 1;
 	    }
@@ -1529,13 +1607,13 @@ static char * formatValue(struct sprintfTag * tag, Header h,
 			 extCache + tag->extNum)) {
 	    count = 1;
 	    type = RPM_STRING_TYPE;	
-	    data = "(none)";
+	    data = _("(none)");
 	}
     } else {
 	if (!headerGetEntry(h, tag->tag, &type, &data, &count)){
 	    count = 1;
 	    type = RPM_STRING_TYPE;	
-	    data = "(none)";
+	    data = _("(none)");
 	}
 
 	mayfree = 1;
@@ -1629,7 +1707,7 @@ static char * formatValue(struct sprintfTag * tag, Header h,
 
       default:
 	val = malloc(20);
-	strcpy(val, "(unknown type)");
+	strcpy(val, _("(unknown type)"));
     }
 
     return val;
@@ -1675,7 +1753,7 @@ static char * singleSprintf(Header h, struct sprintfToken * token,
 	}
 
 	alloced = condNumFormats * 20;
-	val = malloc(alloced);
+	val = malloc(alloced ? alloced : 1);
 	*val = '\0';
 	len = 0;
 
@@ -1718,7 +1796,7 @@ static char * singleSprintf(Header h, struct sprintfToken * token,
 
 	if (numElements == -1) {
 	    val = malloc(20);
-	    strcpy(val, "(none)");
+	    strcpy(val, _("(none)"));
 	} else {
 	    alloced = numElements * token->u.array.numTokens * 20;
 	    val = malloc(alloced);
@@ -1760,7 +1838,25 @@ static struct extensionCache * allocateExtensionCache(
 	    ext++;
     }
 
-    return calloc(i, sizeof(struct extensionCache *));
+    return calloc(i, sizeof(struct extensionCache));
+}
+
+void freeExtensionCache(const struct headerSprintfExtension * extensions,
+		        struct extensionCache * cache) {
+    const struct headerSprintfExtension * ext = extensions;
+    int i = 0;
+
+    while (ext->type != HEADER_EXT_LAST) {
+	if (cache[i].freeit) free(cache[i].data);
+
+	i++;
+	if (ext->type == HEADER_EXT_MORE)
+	    ext = ext->u.more;
+	else
+	    ext++;
+    }
+
+    free(cache);
 }
 
 char * headerSprintf(Header h, const char * origFmt, 
@@ -1809,6 +1905,8 @@ char * headerSprintf(Header h, const char * origFmt,
     }
 
     free(fmtString);
+    freeExtensionCache(extensions, extCache);
+    free(format);
 
     return answer;
 }
@@ -1819,7 +1917,7 @@ static char * octalFormat(int_32 type, const void * data,
 
     if (type != RPM_INT32_TYPE) {
 	val = malloc(20);
-	strcpy(val, "(not a number)");
+	strcpy(val, _("(not a number)"));
     } else {
 	val = malloc(20 + padding);
 	strcat(formatPrefix, "o");
@@ -1839,7 +1937,7 @@ static char * realDateFormat(int_32 type, const void * data,
 
     if (type != RPM_INT32_TYPE) {
 	val = malloc(20);
-	strcpy(val, "(not a number)");
+	strcpy(val, _("(not a number)"));
     } else {
 	val = malloc(50 + padding);
 	strcat(formatPrefix, "s");
