@@ -8,6 +8,11 @@
 #ifndef H_HEADER
 #define H_HEADER
 #include <stdio.h>
+#include <zlib.h>
+
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 #if defined(__alpha__)
 typedef long int int_64;
@@ -45,6 +50,7 @@ enum headerSprintfExtenstionType { HEADER_EXT_LAST = 0, HEADER_EXT_FORMAT,
 typedef char * (*headerTagFormatFunction)(int_32 type, const void * data, 
 					  char * formatPrefix,
 					  int padding, int element);
+/* This is allowed to fail, which indicates the tag doesn't exist */
 typedef int (*headerTagTagFunction)(Header h, int_32 * type, void ** data,
 				       int_32 * count, int * freeData);
 
@@ -66,6 +72,8 @@ extern const struct headerSprintfExtension headerDefaultFormats[];
 /* read and write a header from a file */
 Header headerRead(int fd, int magicp);
 void headerWrite(int fd, Header h, int magicp);
+Header headerGzRead(gzFile fd, int magicp);
+void headerGzWrite(gzFile fd, Header h, int magicp);
 unsigned int headerSizeof(Header h, int magicp);
 
 #define HEADER_MAGIC_NO   0
@@ -95,10 +103,11 @@ char * headerSprintf(Header h, const char * fmt,
    arrays through this function, you probably don't mean to. See
    headerAddI18NString() instead */
 int headerAddEntry(Header h, int_32 tag, int_32 type, void *p, int_32 c);
+/* if there are multiple entries with this tag, the first one gets replaced */
 int headerModifyEntry(Header h, int_32 tag, int_32 type, void *p, int_32 c);
 
-/* For the C locale, lang should be *NULL*. Here are the rules:
-
+/* A NULL lang is interpreted as the C locale.  Here are the rules:
+   
 	1) If the tag isn't in the Header, it's added with the passed string
 	   as a version.
 	2) If the tag occurs multiple times in entry, which tag is affected
@@ -167,4 +176,8 @@ void headerSort(Header h);
 
 #define HEADER_I18NTABLE	100
 
-#endif H_HEADER
+#ifdef __cplusplus
+}
+#endif
+
+#endif /* H_HEADER */
