@@ -43,6 +43,7 @@ struct fileMemory {
     char ** md5s;
     char ** links;
     char ** names;
+    char ** cpioNames;
     struct fileInfo * files;
 };
 
@@ -85,16 +86,17 @@ static int installSources(Header h, char * rootdir, int fd,
 			  char ** specFilePtr, rpmNotifyFunction notify,
 			  char * labelFormat);
 static int markReplacedFiles(rpmdb db, struct replacedFile * replList);
-static int relocateFilelist(Header * hp, char * defaultPrefix, 
-			char * newPrefix, int * relocationSize);
 static int archOkay(Header h);
 static int osOkay(Header h);
 static int ensureOlder(rpmdb db, Header new, int dbOffset);
-static void assembleFileList(Header h, struct fileMemory * mem, 
-			     int * fileCountPtr, struct fileInfo ** files, 
-			     int stripPrefixLength);
+static int assembleFileList(Header h, struct fileMemory * mem, 
+			     int * fileCountPtr, struct fileInfo ** filesPtr, 
+			     int stripPrefixLength,
+			     struct rpmRelocation * rawRelocations,
+			     int allowRandomRelocations);
 static void setFileOwners(Header h, struct fileInfo * files, int fileCount);
 static void freeFileMemory(struct fileMemory fileMem);
+static void trimChangelog(Header h);
 
 /* 0 success */
 /* 1 bad magic */
@@ -139,23 +141,173 @@ static void freeFileMemory(struct fileMemory fileMem) {
     free(fileMem.md5s);
     free(fileMem.links);
     free(fileMem.names);
+    free(fileMem.cpioNames);
 }
 
 /* files should not be preallocated */
-static void assembleFileList(Header h, struct fileMemory * mem, 
+static int assembleFileList(Header h, struct fileMemory * mem, 
 			     int * fileCountPtr, struct fileInfo ** filesPtr, 
-			     int stripPrefixLength) {
+			     int stripPrefixLength,
+			     struct rpmRelocation * rawRelocations,
+			     int allowRandomRelocations) {
     uint_32 * fileFlags;
     uint_32 * fileSizes;
     uint_16 * fileModes;
     struct fileInfo * files;
     struct fileInfo * file;
     int fileCount;
-    int i;
+    int i, j, numRelocations = 0, madeSwap, len, newLen;
+    struct rpmRelocation * relocations = NULL;
+    struct rpmRelocation tmpReloc;
+    struct rpmRelocation * nextReloc;
+    char ** validRelocations = NULL, ** actualRelocations;
+    char ** fileLangs;
+    char * newName, * chptr;
+    int rc;
+    int numValid;
+    char ** languages, ** lang;
+
+    if (rawRelocations) {
+	if (!headerGetEntry(h, RPMTAG_PREFIXES, NULL,
+			    (void **) &validRelocations, &numValid)) {
+	    numValid = 0;
+	}
+
+	for (i = 0; rawRelocations[i].newPath; i++) ;
+	numRelocations = i;
+	relocations = alloca(sizeof(*relocations) * numRelocations);
+
+/* XXX this code assumes the validRelocations array won't
+   have trailing /'s in it */
+
+	for (i = 0; i < numRelocations; i++) {
+	    if (!rawRelocations[i].oldPath) {
+		if (!numValid) {
+		    rpmError(RPMERR_NORELOCATE, 
+			     "package is not relocatable");
+		    return 1;
+		} else if (numValid != 1){
+		    rpmError(RPMERR_NORELOCATE, 
+			     "package has multiple relocatable components");
+		    return 1;
+		}
+		relocations[i].oldPath = 
+		    alloca(strlen(validRelocations[0]) + 1);
+		strcpy(relocations[i].oldPath, validRelocations[0]);
+	    } else {
+		relocations[i].oldPath = 
+		    alloca(strlen(rawRelocations[i].oldPath) + 1);
+		strcpy(relocations[i].oldPath, rawRelocations[i].oldPath);
+		stripTrailingSlashes(relocations[i].oldPath);
+	    }
+
+	    relocations[i].newPath = 
+		alloca(strlen(rawRelocations[i].newPath) + 1);
+	    strcpy(relocations[i].newPath, rawRelocations[i].newPath);
+	    stripTrailingSlashes(relocations[i].newPath);
+
+	    if (!allowRandomRelocations) {
+		for (j = 0; j < numValid; j++) 
+		    if (!strcmp(validRelocations[j],
+				relocations[i].oldPath)) break;
+		if (j == numValid) {
+		    rpmError(RPMERR_BADRELOCATE, "path %s is not relocatable",
+			     relocations[i].oldPath);
+		    return 1;
+		}
+	    }
+	}
+
+	/* stupid bubble sort, but it's probably faster here */
+	for (i = 0; i < numRelocations; i++) {
+	    madeSwap = 0;
+	    for (j = 1; j < numRelocations; j++) {
+		if (strcmp(relocations[j - 1].oldPath, 
+			   relocations[j].oldPath) > 0) {
+		    tmpReloc = relocations[j - 1];
+		    relocations[j - 1] = relocations[j];
+		    relocations[j] = tmpReloc;
+		    madeSwap = 1;
+		}
+	    }
+	    if (!madeSwap) break;
+	}
+
+	if (validRelocations) free(validRelocations);
+    }
+
+    if (headerGetEntry(h, RPMTAG_PREFIXES, NULL,
+			(void **) &validRelocations, &numValid)) {
+	actualRelocations = malloc(sizeof(*actualRelocations) * numValid);
+
+	/* handle the special case of oldPath == NULL, which can only happen
+	   when numValid == 1 (which we've tested for when we build the
+	   relocation table above */
+	for (i = 0; i < numValid; i++) {
+	    for (j = 0; j < numRelocations; j++) {
+		if (!strcmp(validRelocations[i], relocations[j].oldPath)) {
+		    actualRelocations[i] = relocations[j].newPath;
+		    break;
+		}
+	    }
+
+	    if (j == numRelocations)
+		actualRelocations[i] = validRelocations[i];
+	}
+
+	headerAddEntry(h, RPMTAG_INSTPREFIXES, RPM_STRING_ARRAY_TYPE,
+		       (void **) actualRelocations, numValid);
+
+	free(validRelocations);
+	free(actualRelocations);
+    }
 
     headerGetEntry(h, RPMTAG_FILENAMES, NULL, (void **) &mem->names, 
 		   fileCountPtr);
+    /* get this before we start mucking with this info */
+    headerGetEntry(h, RPMTAG_FILENAMES, NULL, (void **) &mem->cpioNames, 
+		   fileCountPtr);
     fileCount = *fileCountPtr;
+
+    if (relocations) {
+	/* go through things backwards so that /usr/local relocations take
+	   precedence over /usr ones */
+	nextReloc = relocations + numRelocations - 1;
+	len = strlen(nextReloc->oldPath);
+	newLen = strlen(nextReloc->newPath);
+	for (i = fileCount - 1; i >= 0 && nextReloc; i--) {
+	    do {
+		rc = strncmp(nextReloc->oldPath, mem->names[i], len);
+		if (rc > 0) {
+		    if (nextReloc == relocations) {
+			nextReloc = 0;
+		    } else {
+			nextReloc--;
+			len = strlen(nextReloc->oldPath);
+			newLen = strlen(nextReloc->newPath);
+		    }
+		}
+	    } while (rc > 0 && nextReloc);
+
+	    if (!rc) {
+		newName = alloca(newLen + strlen(mem->names[i]));
+		strcpy(newName, nextReloc->newPath);
+		strcat(newName, mem->names[i] + len);
+		rpmMessage(RPMMESS_DEBUG, "relocating %s to %s\n",
+			   mem->names[i], newName);
+		mem->names[i] = newName;
+	    } 
+	}
+
+	headerModifyEntry(h, RPMTAG_FILENAMES, RPM_STRING_ARRAY_TYPE, 
+			  mem->names, fileCount);
+	/* make mem->names point to the new data in the header rather
+	   then the old (now modified) data */
+	free(mem->names);
+	headerGetEntry(h, RPMTAG_FILENAMES, NULL, (void **) &mem->names, 
+		       fileCountPtr);
+    }
+
     files = *filesPtr = mem->files = malloc(sizeof(*mem->files) * fileCount);
     
     headerGetEntry(h, RPMTAG_FILEMD5S, NULL, (void **) &mem->md5s, NULL);
@@ -163,6 +315,13 @@ static void assembleFileList(Header h, struct fileMemory * mem,
     headerGetEntry(h, RPMTAG_FILEMODES, NULL, (void **) &fileModes, NULL);
     headerGetEntry(h, RPMTAG_FILESIZES, NULL, (void **) &fileSizes, NULL);
     headerGetEntry(h, RPMTAG_FILELINKTOS, NULL, (void **) &mem->links, NULL);
+    if (!headerGetEntry(h, RPMTAG_FILELANGS, NULL, (void **) &fileLangs, NULL))
+	fileLangs = NULL;
+
+    if ((chptr = getenv("LINGUAS"))) {
+	languages = splitString(chptr, strlen(chptr), ':');
+    } else
+	languages = NULL;
 
     for (i = 0, file = files; i < fileCount; i++, file++) {
 	file->state = RPMFILE_STATE_NORMAL;
@@ -170,13 +329,28 @@ static void assembleFileList(Header h, struct fileMemory * mem,
 	file->install = 1;
 
 	file->relativePath = mem->names[i];
-	file->cpioPath = mem->names[i] + stripPrefixLength;
+	file->cpioPath = mem->cpioNames[i] + stripPrefixLength;
 	file->mode = fileModes[i];
 	file->md5 = mem->md5s[i];
 	file->link = mem->links[i];
 	file->size = fileSizes[i];
 	file->flags = fileFlags[i];
+
+	if (fileLangs && languages && *fileLangs[i]) {
+	    for (lang = languages; *lang; lang++)
+		if (!strcmp(*lang, fileLangs[i])) break;
+	    if (!*lang) {
+		file->install = 0;
+		rpmMessage(RPMMESS_DEBUG, "not installing %s -- linguas\n",
+			   file->relativePath);
+	    }
+	}
     }
+
+    if (fileLangs) free(fileLangs);
+    if (languages) freeSplitString(languages);
+
+    return 0;
 }
 
 static void setFileOwners(Header h, struct fileInfo * files, int fileCount) {
@@ -209,12 +383,55 @@ static void setFileOwners(Header h, struct fileInfo * files, int fileCount) {
     free(fileGroups);
 }
 
+static void trimChangelog(Header h) {
+    int * times;
+    char ** names, ** texts;
+    long numToKeep;
+    char * buf, * end;
+    int count;
+
+    buf = rpmGetVar(RPMVAR_INSTCHANGELOG);
+    if (!buf) return;
+
+    numToKeep = strtol(buf, &end, 10);
+    if (*end) {
+	rpmError(RPMERR_RPMRC, "instchangelog value in rpmrc should be a "
+		    "number, but isn't");
+	return;
+    }
+
+    if (numToKeep < 0) return;
+
+    if (!numToKeep) {
+	headerRemoveEntry(h, RPMTAG_CHANGELOGTIME);
+	headerRemoveEntry(h, RPMTAG_CHANGELOGNAME);
+	headerRemoveEntry(h, RPMTAG_CHANGELOGTEXT);
+	return;
+    }
+
+    if (!headerGetEntry(h, RPMTAG_CHANGELOGTIME, NULL, (void **) &times, 
+			&count) ||
+	count < numToKeep) return;
+    headerGetEntry(h, RPMTAG_CHANGELOGNAME, NULL, (void **) &names, &count);
+    headerGetEntry(h, RPMTAG_CHANGELOGTEXT, NULL, (void **) &texts, &count);
+
+    headerModifyEntry(h, RPMTAG_CHANGELOGTIME, RPM_INT32_TYPE, times, 
+		      numToKeep);
+    headerModifyEntry(h, RPMTAG_CHANGELOGNAME, RPM_STRING_ARRAY_TYPE, names, 
+		      numToKeep);
+    headerModifyEntry(h, RPMTAG_CHANGELOGTEXT, RPM_STRING_ARRAY_TYPE, texts, 
+		      numToKeep);
+
+    free(names);
+    free(texts);
+}
+
 /* 0 success */
 /* 1 bad magic */
 /* 2 error */
-int rpmInstallPackage(char * rootdir, rpmdb db, int fd, char * location, 
-		      int flags, rpmNotifyFunction notify, char * labelFormat,
-		      char * netsharedPath) {
+int rpmInstallPackage(char * rootdir, rpmdb db, int fd,
+		      struct rpmRelocation * relocations,
+		      int flags, rpmNotifyFunction notify, char * labelFormat) {
     int rc, isSource, major, minor;
     char * name, * version, * release;
     Header h;
@@ -234,10 +451,10 @@ int rpmInstallPackage(char * rootdir, rpmdb db, int fd, char * location,
     int * toRemove = NULL;
     int toRemoveAlloced = 1;
     int * intptr = NULL;
-    char * archivePrefix, * tmpPath;
+    char * tmpPath;
     int scriptArg;
     int hasOthers = 0;
-    int relocationSize = 1;		/* strip at least first / for cpio */
+    int stripSize = 1;		/* strip at least first / for cpio */
     uint_32 * archiveSizePtr;
     struct fileMemory fileMem;
     int freeFileMem = 0;
@@ -276,19 +493,6 @@ int rpmInstallPackage(char * rootdir, rpmdb db, int fd, char * location,
     headerGetEntry(h, RPMTAG_VERSION, &type, (void **) &version, &fileCount);
     headerGetEntry(h, RPMTAG_RELEASE, &type, (void **) &release, &fileCount);
 
-    if (!headerGetEntry(h, RPMTAG_DEFAULTPREFIX, &type, (void *)
-			      &defaultPrefix, &fileCount)) {
-	defaultPrefix = NULL;
-    }
-
-    if (location && !defaultPrefix) {
-	rpmError(RPMERR_NORELOCATE, "package %s-%s-%s is not relocatable",
-		      name, version, release);
-	headerFree(h);
-	return 2;
-    } else if (!location && defaultPrefix)
-	location = defaultPrefix;
-
     /* We don't use these entries (and never have) and they are pretty
        misleading. Let's just get rid of them so they don't confuse
        anyone. */
@@ -297,17 +501,6 @@ int rpmInstallPackage(char * rootdir, rpmdb db, int fd, char * location,
     if (headerIsEntry(h, RPMTAG_FILEGROUPNAME))
 	headerRemoveEntry(h, RPMTAG_FILEGIDS);
     
-    if (location) {
-	relocateFilelist(&h, defaultPrefix, location, &relocationSize);
-        headerGetEntry(h, RPMTAG_DEFAULTPREFIX, &type, (void *) &defaultPrefix, 
-		&fileCount);
-	archivePrefix = alloca(strlen(rootdir) + strlen(location) + 2);
-	sprintf(archivePrefix, "%s/%s", rootdir, location);
-    } else {
-	archivePrefix = rootdir;
-	relocationSize = 1;
-    }
-
     if (!(flags & RPMINSTALL_NOARCH) && !archOkay(h)) {
 	rpmError(RPMERR_BADARCH, "package %s-%s-%s is for a different "
 	      "architecture", name, version, release);
@@ -393,7 +586,7 @@ int rpmInstallPackage(char * rootdir, rpmdb db, int fd, char * location,
 		if (rc == 1) continue;		/* no matches */
 
 		rpmMessage(RPMMESS_DEBUG, "package %s is now obsolete and will"
-			   "be removed\n", obsoletes[i]);
+			   " be removed\n", obsoletes[i]);
 
 		toRemoveAlloced += matches.count;
 		j = toRemove ? intptr - toRemove : 0; 
@@ -448,12 +641,43 @@ int rpmInstallPackage(char * rootdir, rpmdb db, int fd, char * location,
 	char ** netsharedPaths;
 	char ** nsp;
 
-	freeFileMem = 1;
-	assembleFileList(h, &fileMem, &fileCount, &files, relocationSize);
+	/* RPM used to allow only "fully relocateable packages", which
+	   were done by making the paths in cpio archive relative to the
+	   relocation point (as RPM just exec()d cpio then). If this is
+	   such a package, undo this and build a proper relocation entry
+	   for it. In any case, we still have to strip the leading / from
+	   the filename to get the one in the cpio archive, as they are
+	   always stored as relative paths */
+	   
+	if (headerGetEntry(h, RPMTAG_DEFAULTPREFIX, NULL, (void *)
+				  &defaultPrefix, NULL)) {
+	    /* a trailing '/' in the defaultPrefix would confuse us */
+	    defaultPrefix = strcpy(alloca(strlen(defaultPrefix) + 1), 
+				   defaultPrefix);
+	    stripTrailingSlashes(defaultPrefix);
+	    stripSize = strlen(defaultPrefix) + 1;
 
-	if (netsharedPath) 
-	    netsharedPaths = splitString(netsharedPath, 
-					 strlen(netsharedPath), ':');
+	    headerAddEntry(h, RPMTAG_PREFIXES, RPM_STRING_ARRAY_TYPE,
+			   &defaultPrefix, 1); 
+	} else {
+	    stripSize = 1;
+	    defaultPrefix = NULL;
+	}
+
+	if (assembleFileList(h, &fileMem, &fileCount, &files, stripSize,
+			     relocations, flags & RPMINSTALL_FORCERELOCATE)) {
+	    if (rootdir) {
+		chroot(".");
+		chdir(currDir);
+	    }
+	    if (replacedList) free(replacedList);
+	    return 2;
+	}
+
+	freeFileMem = 1;
+
+	if ((tmpPath = rpmGetVar(RPMVAR_NETSHAREDPATH)))
+	    netsharedPaths = splitString(tmpPath, strlen(tmpPath), ':');
 	else
 	    netsharedPaths = NULL;
 
@@ -462,7 +686,7 @@ int rpmInstallPackage(char * rootdir, rpmdb db, int fd, char * location,
 	   instHandleSharedFiles() below will take care of the problem */
 	for (i = 0; i < fileCount; i++) {
 	    /* netsharedPaths are not relative to the current root (though 
-	       they do need to take the package prefix into account */
+	       they do need to take package relocations into account) */
 	    for (nsp = netsharedPaths; nsp && *nsp; nsp++) {
 		j = strlen(*nsp);
 		if (!strncmp(files[i].relativePath, *nsp, j) &&
@@ -503,6 +727,8 @@ int rpmInstallPackage(char * rootdir, rpmdb db, int fd, char * location,
 	    }
 	}
 
+	if (netsharedPaths) freeSplitString(netsharedPaths);
+
 	rc = instHandleSharedFiles(db, otherOffset, files, fileCount, 
 				   toRemove, &replacedList, flags);
 
@@ -532,8 +758,8 @@ int rpmInstallPackage(char * rootdir, rpmdb db, int fd, char * location,
     }
 
     rpmMessage(RPMMESS_DEBUG, "running preinstall script (if any)\n");
-    if (runScript("/", h, RPMTAG_PREIN, RPMTAG_PREINPROG, scriptArg, 
-		  flags & RPMINSTALL_NOSCRIPTS, 0)) {
+    if (runInstScript("/", h, RPMTAG_PREIN, RPMTAG_PREINPROG, scriptArg, 
+		      flags & RPMINSTALL_NOSCRIPTS, 0)) {
 	if (replacedList) free(replacedList);
 	if (freeFileMem) freeFileMemory(fileMem);
 
@@ -651,22 +877,31 @@ int rpmInstallPackage(char * rootdir, rpmdb db, int fd, char * location,
 	    return 2;
 	}
 
-	if (files) {
-	    fileStates = malloc(sizeof(*fileStates) * fileCount);
-	    for (i = 0; i < fileCount; i++)
-		fileStates[i] = files[i].state;
+	fileStates = malloc(sizeof(*fileStates) * fileCount);
+	for (i = 0; i < fileCount; i++)
+	    fileStates[i] = files[i].state;
 
+	headerAddEntry(h, RPMTAG_FILESTATES, RPM_CHAR_TYPE, fileStates, 
+			fileCount);
+
+	free(fileStates);
+	if (freeFileMem) freeFileMemory(fileMem);
+    } else if (flags & RPMINSTALL_JUSTDB) {
+	char ** fileNames;
+
+	if (headerGetEntry(h, RPMTAG_FILENAMES, NULL, (void **) &fileNames,
+		           &fileCount)) {
+	    fileStates = malloc(sizeof(*fileStates) * fileCount);
+	    memset(fileStates, RPMFILE_STATE_NOTINSTALLED, fileCount);
 	    headerAddEntry(h, RPMTAG_FILESTATES, RPM_CHAR_TYPE, fileStates, 
 			    fileCount);
-
 	    free(fileStates);
-	    if (freeFileMem) freeFileMemory(fileMem);
 	}
-
-	installTime = time(NULL);
-	headerAddEntry(h, RPMTAG_INSTALLTIME, RPM_INT32_TYPE, &installTime, 1);
     }
 
+    installTime = time(NULL);
+    headerAddEntry(h, RPMTAG_INSTALLTIME, RPM_INT32_TYPE, &installTime, 1);
+
     if (rootdir) {
 	chroot(".");
 	chdir(currDir);
@@ -679,6 +914,8 @@ int rpmInstallPackage(char * rootdir, rpmdb db, int fd, char * location,
 	if (rc) return rc;
     }
 
+    trimChangelog(h);
+
     /* if this package has already been installed, remove it from the database
        before adding the new one */
     if (otherOffset) {
@@ -692,8 +929,19 @@ int rpmInstallPackage(char * rootdir, rpmdb db, int fd, char * location,
 
     rpmMessage(RPMMESS_DEBUG, "running postinstall script (if any)\n");
 
-    if (runScript(rootdir, h, RPMTAG_POSTIN, RPMTAG_POSTINPROG, scriptArg,
-		  flags & RPMINSTALL_NOSCRIPTS, 0)) {
+    if (runInstScript(rootdir, h, RPMTAG_POSTIN, RPMTAG_POSTINPROG, scriptArg,
+		      flags & RPMINSTALL_NOSCRIPTS, 0)) {
+	return 2;
+    }
+
+    /* Run triggers this package sets off */
+    if (runTriggers(rootdir, db, RPMSENSE_TRIGGERIN, h, 0)) {
+	return 2;
+    }
+
+    /* Run triggers in this package which are set off by other things in
+       the database. */
+    if (runImmedTriggers(rootdir, db, RPMSENSE_TRIGGERIN, h, 0)) {
 	return 2;
     }
 
@@ -782,6 +1030,7 @@ static int installArchive(int fd, struct fileInfo * files,
 			    ((notify && archiveSize) || specFile) ? 
 				callback : NULL, 
 			    &info, &failedFile);
+    gzclose(stream);
 
     if (rc) {
 	/* this would probably be a good place to check if disk space
@@ -1000,7 +1249,7 @@ static int instHandleSharedFiles(rpmdb db, int ignoreOffset,
     for (i = 0; i < sharedCount; i++) {
 	/* if a file isn't going to be installed it doesn't matter who
 	   it's shared with */
-	if (!files[sharedList[i].mainFileNumber].action == SKIP) continue;
+	if (files[sharedList[i].mainFileNumber].action == SKIP) continue;
 
 	if (secOffset != sharedList[i].secRecOffset) {
 	    if (secOffset) {
@@ -1193,7 +1442,7 @@ static int installSources(Header h, char * rootdir, int fd,
 
     if (h && headerIsEntry(h, RPMTAG_FILENAMES)) {
 	/* we can't remap v1 packages */
-	assembleFileList(h, &fileMem, &fileCount, &files, 0);
+	assembleFileList(h, &fileMem, &fileCount, &files, 0, NULL, 0);
 
 	for (i = 0; i < fileCount; i++) {
 	    files[i].relativePath = files[i].relativePath;
@@ -1428,70 +1677,6 @@ enum fileTypes whatis(short mode) {
     return result;
 }
 
-static int relocateFilelist(Header * hp, char * defaultPrefix, 
-			    char * newPrefix, int * relocationLength) {
-    Header h = *hp;
-    char ** newFileList, ** fileList;
-    int fileCount, i;
-    int defaultPrefixLength;
-    int newPrefixLength;
-
-    /* a trailing '/' in the defaultPrefix or in the newPrefix would really
-       confuse us */
-    defaultPrefix = strcpy(alloca(strlen(defaultPrefix) + 1), defaultPrefix);
-    stripTrailingSlashes(defaultPrefix);
-    newPrefix = strcpy(alloca(strlen(newPrefix) + 1), newPrefix);
-    stripTrailingSlashes(newPrefix);
-
-    rpmMessage(RPMMESS_DEBUG, "relocating files from %s to %s\n", defaultPrefix,
-			 newPrefix);
-
-    if (!strcmp(newPrefix, defaultPrefix)) {
-	headerAddEntry(h, RPMTAG_INSTALLPREFIX, RPM_STRING_TYPE, 
-			defaultPrefix, 1);
-	*relocationLength = strlen(defaultPrefix) + 1;
-	return 0;
-    }
-
-    defaultPrefixLength = strlen(defaultPrefix);
-    newPrefixLength = strlen(newPrefix);
-
-    /* packages may have empty filelists */
-    if (!headerGetEntry(h, RPMTAG_FILENAMES, NULL, (void *) &fileList, 
-			&fileCount))
-	return 0;
-    else if (!fileCount)
-	return 0;
-
-    newFileList = alloca(sizeof(char *) * fileCount);
-    for (i = 0; i < fileCount; i++) {
-	if (!strcmp(fileList[i], defaultPrefix)) {
-	    /* special case as there is no '/' on this */
-	    newFileList[i] = newPrefix;
-	} else if (!strncmp(fileList[i], defaultPrefix, defaultPrefixLength)) {
-	    newFileList[i] = alloca(strlen(fileList[i]) + newPrefixLength -
-				 defaultPrefixLength + 2);
-	    sprintf(newFileList[i], "%s/%s", newPrefix, 
-		    fileList[i] + defaultPrefixLength + 1);
-	} else {
-	    rpmMessage(RPMMESS_DEBUG, "BAD - unprefixed file in relocatable "
-			"package");
-	    newFileList[i] = alloca(strlen(fileList[i]) - 
-					defaultPrefixLength + 2);
-	    sprintf(newFileList[i], "/%s", fileList[i] + 
-			defaultPrefixLength + 1);
-	}
-    }
-
-    headerModifyEntry(h, RPMTAG_FILENAMES, RPM_STRING_ARRAY_TYPE, 
-			newFileList, fileCount);
-    headerAddEntry(h, RPMTAG_INSTALLPREFIX, RPM_STRING_TYPE, newPrefix, 1);
-
-    *relocationLength = newPrefixLength + 1;
-
-    return 0;
-}
-
 static int archOkay(Header h) {
     int_8 * pkgArchNum;
     void * pkgArch;
