@@ -1,10 +1,12 @@
 #include <stdarg.h>
 #include <stdio.h>
 
+#include "intl.h"
 #include "messages.h"
 #include "rpmlib.h"
+#include "intl.h"
 
-static minLevel = RPMMESS_NORMAL;
+static int minLevel = RPMMESS_NORMAL;
 
 void rpmIncreaseVerbosity(void) {
     minLevel--;
@@ -48,19 +50,19 @@ void rpmMessage(int level, char * format, ...) {
 	    break;
 
 	  case RPMMESS_WARNING:
-	    fprintf(stderr, "warning: ");
+	    fprintf(stderr, _("warning: "));
 	    vfprintf(stderr, format, args);
 	    fflush(stderr);
 	    break;
 
 	  case RPMMESS_ERROR:
-	    fprintf(stderr, "error: ");
+	    fprintf(stderr, _("error: "));
 	    vfprintf(stderr, format, args);
 	    fflush(stderr);
 	    break;
 
 	  case RPMMESS_FATALERROR:
-	    fprintf(stderr, "fatal error: ");
+	    fprintf(stderr, _("fatal error: "));
 	    vfprintf(stderr, format, args);
 	    fflush(stderr);
 	    exit(1);
