@@ -23,7 +23,7 @@
 #include "rpmlib.h"
 #include "messages.h"
 
-char * RPMVERSION = VERSION;
+char * RPMVERSION = VERSION;	/* just to put a marker in librpm.a */
 
 char ** splitString(char * str, int length, char sep) {
     char * s, * source, * dest;
@@ -336,7 +336,7 @@ int makeTempFile(char * prefix, char ** fnptr, int * fdptr) {
     int fd;
     int ran;
     char * tmpdir = rpmGetVar(RPMVAR_TMPPATH);
-    struct stat sb;
+    struct stat sb, sb2;
 
     if (!prefix) prefix = "";
 
@@ -346,16 +346,21 @@ int makeTempFile(char * prefix, char ** fnptr, int * fdptr) {
     ran = rand() % 100000;
     do {
 	sprintf(fn, "%s%s/rpm-tmp.%d", prefix, tmpdir, ran++);
-    } while (!access(fn, X_OK));
-    
-    fd = open(fn, O_CREAT | O_RDWR | O_EXCL, 0700);
+	fd = open(fn, O_CREAT | O_RDWR | O_EXCL, 0700);
+    } while (fd < 0);
 
-    if (fd < 0) {
+    if (!stat(fn, &sb) && S_ISLNK(sb.st_mode)) {
 	rpmError(RPMERR_SCRIPT, _("error creating temporary file %s"), fn);
 	return 1;
     }
 
-    if (!stat(fn, &sb) && S_ISLNK(sb.st_mode)) {
+    if (sb.st_nlink != 1) {
+	rpmError(RPMERR_SCRIPT, _("error creating temporary file %s"), fn);
+	return 1;
+    }
+
+    fstat(fd, &sb2);
+    if (sb2.st_ino != sb.st_ino || sb2.st_dev != sb.st_dev) {
 	rpmError(RPMERR_SCRIPT, _("error creating temporary file %s"), fn);
 	return 1;
     }
