@@ -16,6 +16,7 @@
 #include "rpmlib.h"
 #include "rpmlead.h"
 #include "tread.h"
+#include "intl.h"
 
 /* The lead needs to be 8 byte aligned */
 
@@ -45,7 +46,7 @@ int writeLead(int fd, struct rpmlead *lead)
 int readLead(int fd, struct rpmlead *lead)
 {
     if (timedRead(fd, lead, sizeof(*lead)) != sizeof(*lead)) {
-	rpmError(RPMERR_READERROR, "read failed: %s (%d)", strerror(errno), 
+	rpmError(RPMERR_READERROR, _("read failed: %s (%d)"), strerror(errno), 
 	      errno);
 	return 1;
     }
