@@ -3,12 +3,20 @@
 
 /* This is the *only* module users of rpmlib should need to include */
 
+#ifdef HAVE_DB_185_H
+#include <db_185.h>
+#else
 #include <db.h>
+#endif
 
 /* it shouldn't need these :-( */
 #include "dbindex.h"
 #include "header.h"
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 int rpmReadPackageInfo(int fd, Header * signatures, Header * hdr);
 int rpmReadPackageHeader(int fd, Header * hdr, int * isSource, int * major,
 			 int * minor);
@@ -66,8 +74,8 @@ extern const struct headerSprintfExtension rpmHeaderFormats[];
 #define RPMTAG_ROOT                     1038
 #define RPMTAG_FILEUSERNAME             1039
 #define RPMTAG_FILEGROUPNAME            1040
-#define RPMTAG_EXCLUDE                  1041 /* not used - depricated */
-#define RPMTAG_EXCLUSIVE                1042 /* not used - depricated */
+#define RPMTAG_EXCLUDE                  1041 /* internal - depricated */
+#define RPMTAG_EXCLUSIVE                1042 /* internal - depricated */
 #define RPMTAG_ICON                     1043
 #define RPMTAG_SOURCERPM                1044
 #define RPMTAG_FILEVERIFYFLAGS          1045
@@ -81,20 +89,20 @@ extern const struct headerSprintfExtension rpmHeaderFormats[];
 #define RPMTAG_CONFLICTFLAGS            1053
 #define RPMTAG_CONFLICTNAME             1054
 #define RPMTAG_CONFLICTVERSION          1055
-#define RPMTAG_DEFAULTPREFIX            1056
+#define RPMTAG_DEFAULTPREFIX            1056 /* internal - deprecated */
 #define RPMTAG_BUILDROOT                1057
-#define RPMTAG_INSTALLPREFIX		1058
+#define RPMTAG_INSTALLPREFIX		1058 /* internal - deprecated */
 #define RPMTAG_EXCLUDEARCH              1059
 #define RPMTAG_EXCLUDEOS                1060
 #define RPMTAG_EXCLUSIVEARCH            1061
 #define RPMTAG_EXCLUSIVEOS              1062
 #define RPMTAG_AUTOREQPROV              1063 /* internal */
 #define RPMTAG_RPMVERSION		1064
-#define RPMTAG_TRIGGERSCRIPTS           1065 /* internal */
-#define RPMTAG_TRIGGERNAME              1066 /* internal */
-#define RPMTAG_TRIGGERVERSION           1067 /* internal */
-#define RPMTAG_TRIGGERFLAGS             1068 /* internal */
-#define RPMTAG_TRIGGERINDEX             1069 /* internal */
+#define RPMTAG_TRIGGERSCRIPTS           1065
+#define RPMTAG_TRIGGERNAME              1066
+#define RPMTAG_TRIGGERVERSION           1067
+#define RPMTAG_TRIGGERFLAGS             1068
+#define RPMTAG_TRIGGERINDEX             1069
 #define RPMTAG_VERIFYSCRIPT             1079
 #define RPMTAG_CHANGELOGTIME            1080
 #define RPMTAG_CHANGELOGNAME            1081
@@ -108,13 +116,20 @@ extern const struct headerSprintfExtension rpmHeaderFormats[];
 #define RPMTAG_BUILDARCHS               1089
 #define RPMTAG_OBSOLETES                1090
 #define RPMTAG_VERIFYSCRIPTPROG         1091
-#define RPMTAG_TRIGGERSCRIPTPROGS       1092
+#define RPMTAG_TRIGGERSCRIPTPROG        1092
 #define RPMTAG_DOCDIR                   1093 /* internal */
 #define RPMTAG_COOKIE                   1094
 #define RPMTAG_FILEDEVICES              1095
 #define RPMTAG_FILEINODES               1096
 #define RPMTAG_FILELANGS                1097
-
+#define RPMTAG_PREFIXES                 1098
+#define RPMTAG_INSTPREFIXES             1099
+#define RPMTAG_TRIGGERIN                1100 /* internal */
+#define RPMTAG_TRIGGERUN                1101 /* internal */
+#define RPMTAG_TRIGGERPOSTUN            1102 /* internal */
+#define RPMTAG_AUTOREQ                  1103 /* internal */
+#define RPMTAG_AUTOPROV                 1104 /* internal */
+    
 #define RPMTAG_EXTERNAL_TAG		1000000
 
 #define RPMFILE_STATE_NORMAL 		0
@@ -143,6 +158,7 @@ extern const struct headerSprintfExtension rpmHeaderFormats[];
 #define RPMINSTALL_ALLFILES             (1 << 9)
 #define RPMINSTALL_JUSTDB               (1 << 10)
 #define RPMINSTALL_KEEPOBSOLETE         (1 << 11)
+#define RPMINSTALL_FORCERELOCATE        (1 << 12)
 
 #define RPMUNINSTALL_TEST               (1 << 0)
 #define RPMUNINSTALL_NOSCRIPTS          (1 << 1)
@@ -171,8 +187,11 @@ extern const struct headerSprintfExtension rpmHeaderFormats[];
 #define RPMSENSE_SENSEMASK       15       /* Mask to get senses, ie serial, */
                                           /* less, greater, equal.          */
 
-#define RPMSENSE_TRIGGER_IN      (1 << 16)
-#define RPMSENSE_TRIGGER_UN      (1 << 17)
+#define RPMSENSE_TRIGGERIN       (1 << 16)
+#define RPMSENSE_TRIGGERUN       (1 << 17)
+#define RPMSENSE_TRIGGERPOSTUN   (1 << 18)
+#define RPMSENSE_TRIGGER         (RPMSENSE_TRIGGERIN | RPMSENSE_TRIGGERUN | \
+                                  RPMSENSE_TRIGGERPOSTUN)
 
 /* Stuff for maintaining "variables" like SOURCEDIR, BUILDDIR, etc */
 
@@ -215,8 +234,12 @@ extern const struct headerSprintfExtension rpmHeaderFormats[];
 #define RPMVAR_GZIPBIN     		36
 #define RPMVAR_RPMFILENAME     		37
 #define RPMVAR_PROVIDES     		38
+#define RPMVAR_BUILDSHELL               39
+#define RPMVAR_INSTCHANGELOG            40
+#define RPMVAR_BZIP2BIN     		41
+#define RPMVAR_LANGPATT                 42
 
-#define RPMVAR_NUM			39     /* number of RPMVAR entries */
+#define RPMVAR_NUM			43     /* number of RPMVAR entries */
 
 char * rpmGetVar(int var);
 int rpmGetBooleanVar(int var);
@@ -242,6 +265,7 @@ void rpmSetTables(int archTable, int osTable);  /* only used by build code */
 /* if either are NULL, they are set to the default value (munged uname())
    pushed through a translation table (if appropriate) */
 void rpmSetMachine(char * arch, char * os);
+void rpmGetMachine(char **arch, char **os);
 
 /** **/
 
@@ -267,17 +291,24 @@ int rpmdbFindPackage(rpmdb db, char * name, dbiIndexSet * matches);
 int rpmdbFindByProvides(rpmdb db, char * provides, dbiIndexSet * matches);
 int rpmdbFindByRequiredBy(rpmdb db, char * requires, dbiIndexSet * matches);
 int rpmdbFindByConflicts(rpmdb db, char * conflicts, dbiIndexSet * matches);
+int rpmdbFindByTriggeredBy(rpmdb db, char * package, dbiIndexSet * matches);
 
 /* these are just convience functions */
 int rpmdbFindByLabel(rpmdb db, char * label, dbiIndexSet * matches);
 int rpmdbFindByHeader(rpmdb db, Header h, dbiIndexSet * matches);
 
+/* we pass these aroung as an array with a sentinel */
+struct rpmRelocation {
+    char * oldPath;	/* NULL here evals to RPMTAG_DEFAULTPREFIX, this */
+    char * newPath;     /* odd behavior is only for backwards compatibility */
+};
+
 int rpmInstallSourcePackage(char * root, int fd, char ** specFile,
 			    rpmNotifyFunction notify, char * labelFormat,
 			    char ** cookie);
-int rpmInstallPackage(char * rootdir, rpmdb db, int fd, char * prefix, 
-		      int flags, rpmNotifyFunction notify, char * labelFormat,
-		      char * netsharedPath);
+int rpmInstallPackage(char * rootdir, rpmdb db, int fd,
+		      struct rpmRelocation * relocations,
+		      int flags, rpmNotifyFunction notify, char * labelFormat);
 int rpmVersionCompare(Header first, Header second);
 int rpmRemovePackage(char * root, rpmdb db, unsigned int offset, int flags);
 int rpmdbRebuild(char * root);
@@ -290,6 +321,7 @@ typedef struct rpmDependencyCheck * rpmDependencies;
 
 struct rpmDependencyConflict {
     char * byName, * byVersion, * byRelease;
+    Header byHeader;
     /* these needs fields are misnamed -- they are used for the package
        which isn't needed as well */
     char * needsName, * needsVersion;
@@ -408,7 +440,7 @@ rpmErrorCallBackType rpmErrorSetCallback(rpmErrorCallBackType);
 #define RPMERR_BADARCH          -29     /* bad architecture or arch mismatch */
 #define RPMERR_CREATE		-30	/* failed to create a file */
 #define RPMERR_NOSPACE		-31	/* out of disk space */
-#define RPMERR_NORELOCATE	-32	/* tried to relocate improper package */
+#define RPMERR_NORELOCATE	-32	/* tried to do improper relocatation */
 #define RPMERR_BADOS            -33     /* bad architecture or arch mismatch */
 #define RPMMESS_BACKUP          -34     /* backup made during [un]install */
 #define RPMERR_MTAB		-35	/* failed to read mount table */
@@ -416,6 +448,7 @@ rpmErrorCallBackType rpmErrorSetCallback(rpmErrorCallBackType);
 #define RPMERR_BADDEV		-37	/* file on device not listed in mtab */
 #define RPMMESS_ALTNAME         -38     /* file written as .rpmnew */
 #define RPMMESS_PREREQLOOP      -39     /* loop in prerequisites */
+#define RPMERR_BADRELOCATE      -40     /* bad relocation was specified */
 
 /* spec.c build.c pack.c */
 #define RPMERR_UNMATCHEDIF      -107    /* unclosed %ifarch or %ifos */
@@ -474,4 +507,9 @@ int rpmGetFilesystemList(char *** listptr, int * num);
 int rpmGetFilesystemUsage(char ** filelist, int_32 * fssizes, int numFiles,
 			  uint_32 ** usagesPtr, int flags);
 
+#ifdef __cplusplus
+}
+#endif
+
+    
 #endif
