@@ -17,9 +17,11 @@
 #include <sys/utsname.h>
 #include <unistd.h>
 
+#include "intl.h"
 #include "messages.h"
 #include "misc.h"
 #include "rpmlib.h"
+#include "intl.h"
 
 #if HAVE_SYS_SYSTEMCFG_H
 #include <sys/systemcfg.h>
@@ -99,6 +101,8 @@ static struct tableType tables[RPM_MACHTABLE_COUNT] = {
 static struct rpmOption optionTable[] = {
     { "builddir",		RPMVAR_BUILDDIR,		0, 0 },
     { "buildroot",              RPMVAR_BUILDROOT,               0, 0 },
+    { "buildshell",             RPMVAR_BUILDSHELL,              0, 0 },
+    { "bzip2bin",		RPMVAR_BZIP2BIN,		0, 1 },
     { "dbpath",			RPMVAR_DBPATH,			0, 1 },
     { "defaultdocdir",		RPMVAR_DEFAULTDOCDIR,		0, 0 },
     { "distribution",		RPMVAR_DISTRIBUTION,		0, 0 },
@@ -107,6 +111,8 @@ static struct rpmOption optionTable[] = {
     { "ftpport",		RPMVAR_FTPPORT,			0, 0 },
     { "ftpproxy",		RPMVAR_FTPPROXY,		0, 0 },
     { "gzipbin",		RPMVAR_GZIPBIN,			0, 1 },
+    { "instchangelog",		RPMVAR_INSTCHANGELOG,		0, 0 },
+    { "langpatt",               RPMVAR_LANGPATT,                0, 0 },
     { "messagelevel",		RPMVAR_MESSAGELEVEL,		0, 0 },
     { "netsharedpath",		RPMVAR_NETSHAREDPATH,		0, 0 },
     { "optflags",		RPMVAR_OPTFLAGS,		1, 0 },
@@ -117,7 +123,8 @@ static struct rpmOption optionTable[] = {
     { "require_distribution",	RPMVAR_REQUIREDISTRIBUTION,	0, 0 },
     { "require_icon",		RPMVAR_REQUIREICON,		0, 0 },
     { "require_vendor",		RPMVAR_REQUIREVENDOR,		0, 0 },
-    { "root",			RPMVAR_ROOT,			0, 0 },
+/* root is obsolete - use buildroot instead                             */
+/*    { "root",			RPMVAR_ROOT,			0, 0 }, */
     { "rpmdir",			RPMVAR_RPMDIR,			0, 0 },
     { "rpmfilename",		RPMVAR_RPMFILENAME,		0, 1 },
     { "signature",		RPMVAR_SIGTYPE,			0, 0 },
@@ -203,10 +210,10 @@ static int machCompatCacheAdd(char * name, char * fn, int linenum,
     chptr = name;
     while (*chptr && *chptr != ':') chptr++;
     if (!*chptr) {
-	rpmError(RPMERR_RPMRC, "missing second ':' at %s:%d", fn, linenum);
+	rpmError(RPMERR_RPMRC, _("missing second ':' at %s:%d"), fn, linenum);
 	return 1;
     } else if (chptr == name) {
-	rpmError(RPMERR_RPMRC, "missing architecture name at %s:%d", fn, 
+	rpmError(RPMERR_RPMRC, _("missing architecture name at %s:%d"), fn, 
 			     linenum);
 	return 1;
     }
@@ -306,7 +313,7 @@ static void machCacheEntryVisit(struct machCache * cache,
     for (i = 0; i < entry->count; i++) {
 	machCacheEntryVisit(cache, table, entry->equivs[i], distance + 1);
     }
-};
+}
 
 static void machFindEquivs(struct machCache * cache, 
 			     struct machEquivTable * table,
@@ -343,18 +350,18 @@ static int addCanon(struct canonEntry **table, int *tableLen, char *line,
     t->short_name = strtok(NULL, " \t");
     s = strtok(NULL, " \t");
     if (! (t->name && t->short_name && s)) {
-	rpmError(RPMERR_RPMRC, "Incomplete data line at %s:%d", fn, lineNum);
+	rpmError(RPMERR_RPMRC, _("Incomplete data line at %s:%d"), fn, lineNum);
 	return RPMERR_RPMRC;
     }
     if (strtok(NULL, " \t")) {
-	rpmError(RPMERR_RPMRC, "Too many args in data line at %s:%d",
+	rpmError(RPMERR_RPMRC, _("Too many args in data line at %s:%d"),
 	      fn, lineNum);
 	return RPMERR_RPMRC;
     }
 
     t->num = strtoul(s, &s1, 10);
     if ((*s1) || (s1 == s) || (t->num == ULONG_MAX)) {
-	rpmError(RPMERR_RPMRC, "Bad arch/os number: %s (%s:%d)", s,
+	rpmError(RPMERR_RPMRC, _("Bad arch/os number: %s (%s:%d)"), s,
 	      fn, lineNum);
 	return(RPMERR_RPMRC);
     }
@@ -387,11 +394,12 @@ static int addDefault(struct defaultEntry **table, int *tableLen, char *line,
     t->name = strtok(line, ": \t");
     t->defName = strtok(NULL, " \t");
     if (! (t->name && t->defName)) {
-	rpmError(RPMERR_RPMRC, "Incomplete default line at %s:%d", fn, lineNum);
+	rpmError(RPMERR_RPMRC, _("Incomplete default line at %s:%d"), 
+		 fn, lineNum);
 	return RPMERR_RPMRC;
     }
     if (strtok(NULL, " \t")) {
-	rpmError(RPMERR_RPMRC, "Too many args in default line at %s:%d",
+	rpmError(RPMERR_RPMRC, _("Too many args in default line at %s:%d"),
 	      fn, lineNum);
 	return RPMERR_RPMRC;
     }
@@ -461,6 +469,7 @@ static void setDefaults(void) {
     rpmSetVar(RPMVAR_SIGTYPE, "none");
     rpmSetVar(RPMVAR_DEFAULTDOCDIR, "/usr/doc");
     rpmSetVar(RPMVAR_TOPDIR, "/usr/src/redhat");
+    rpmSetVar(RPMVAR_BUILDSHELL, "/bin/sh");
 }
 
 int rpmReadRC(char * file) {
@@ -476,12 +485,12 @@ int rpmReadRC(char * file) {
     }
 
     fd = open(LIBRPMRC_FILENAME, O_RDONLY);
-    if (fd > 0) {
+    if (fd >= 0) {
 	rc = doReadRC(fd, LIBRPMRC_FILENAME);
 	close(fd);
 	if (rc) return rc;
     } else {
-	rpmError(RPMERR_RPMRC, "Unable to open %s for reading: %s.", 
+	rpmError(RPMERR_RPMRC, _("Unable to open %s for reading: %s."), 
 		 LIBRPMRC_FILENAME, strerror(errno));
 	return 1;
     }
@@ -492,12 +501,12 @@ int rpmReadRC(char * file) {
 	fn = "/etc/rpmrc";
 
     fd = open(fn, O_RDONLY);
-    if (fd > 0) {
+    if (fd >= 0) {
 	rc = doReadRC(fd, fn);
 	close(fd);
 	if (rc) return rc;
     } else if (file) {
-	rpmError(RPMERR_RPMRC, "Unable to open %s for reading: %s.", file,
+	rpmError(RPMERR_RPMRC, _("Unable to open %s for reading: %s."), file,
 		 strerror(errno));
 	return 1;
     }
@@ -509,7 +518,7 @@ int rpmReadRC(char * file) {
 	    strcpy(fn, home);
 	    strcat(fn, "/.rpmrc");
 	    fd = open(fn, O_RDONLY);
-	    if (fd > 0) {
+	    if (fd >= 0) {
 		rc |= doReadRC(fd, fn);
 		close(fd);
 		if (rc) return rc;
@@ -540,7 +549,7 @@ static int doReadRC(int fd, char * filename) {
     fstat(fd, &sb);
     next = buf = alloca(sb.st_size + 2);
     if (read(fd, buf, sb.st_size) != sb.st_size) {
-	rpmError(RPMERR_RPMRC, "Failed to read %s: %s.", filename,
+	rpmError(RPMERR_RPMRC, _("Failed to read %s: %s."), filename,
 		 strerror(errno));
 	return 1;
     }
@@ -570,7 +579,8 @@ static int doReadRC(int fd, char * filename) {
 	}
 
 	if (*chptr != ':') {
-	    rpmError(RPMERR_RPMRC, "missing ':' at %s:%d", filename, linenum);
+	    rpmError(RPMERR_RPMRC, _("missing ':' at %s:%d"), 
+		     filename, linenum);
 	    return 1;
 	}
 
@@ -585,7 +595,7 @@ static int doReadRC(int fd, char * filename) {
 	    while (isspace(*start) && *start) start++;
 
 	    if (! *start) {
-		rpmError(RPMERR_RPMRC, "missing argument for %s at %s:%d", 
+		rpmError(RPMERR_RPMRC, _("missing argument for %s at %s:%d"), 
 		      option->name, filename, linenum);
 		return 1;
 	    }
@@ -596,7 +606,7 @@ static int doReadRC(int fd, char * filename) {
 
 		if (!*chptr) {
 		    rpmError(RPMERR_RPMRC, 
-				"missing architecture for %s at %s:%d", 
+				_("missing architecture for %s at %s:%d"), 
 			  	option->name, filename, linenum);
 		    return 1;
 		}
@@ -606,7 +616,7 @@ static int doReadRC(int fd, char * filename) {
 		while (isspace(*chptr) && *chptr) chptr++;
 		if (!*chptr) {
 		    rpmError(RPMERR_RPMRC, 
-				"missing argument for %s at %s:%d", 
+				_("missing argument for %s at %s:%d"), 
 			  	option->name, filename, linenum);
 		    return 1;
 		}
@@ -649,7 +659,7 @@ static int doReadRC(int fd, char * filename) {
 	    }
 
 	    if (!gotit) {
-		rpmError(RPMERR_RPMRC, "bad option '%s' at %s:%d", 
+		rpmError(RPMERR_RPMRC, _("bad option '%s' at %s:%d"), 
 			    start, filename, linenum);
 	    }
 	}
@@ -677,37 +687,45 @@ static void defaultMachine(char ** arch, char ** os) {
 	while (*chptr++)
 	    if (*chptr == '/') *chptr = '-';
 
+#	if defined(__MIPSEL__) || defined(__MIPSEL) || defined(_MIPSEL)
+	    /* little endian */
+	    strcpy(un.machine, "mipsel");
+#	elif defined(__MIPSEB__) || defined(__MIPSEB) || defined(_MIPSEB)
+	   /* big endian */
+		strcpy(un.machine, "mipseb");
+#	endif
+
 	#if defined(__hpux) && defined(_SC_CPU_VERSION)
 	{
 	    int cpu_version = sysconf(_SC_CPU_VERSION);
 
-	    #if defined(CPU_HP_MC68020)
+#	    if defined(CPU_HP_MC68020)
 		if (cpu_version == CPU_HP_MC68020)
 		    strcpy(un.machine, "m68k");
-	    #endif
-	    #if defined(CPU_HP_MC68030)
+#	    endif
+#	    if defined(CPU_HP_MC68030)
 		if (cpu_version == CPU_HP_MC68030)
 		    strcpy(un.machine, "m68k");
-	    #endif
-	    #if defined(CPU_HP_MC68040)
+#	    endif
+#	    if defined(CPU_HP_MC68040)
 		if (cpu_version == CPU_HP_MC68040)
 		    strcpy(un.machine, "m68k");
-	    #endif
+#	    endif
 
-	    #if defined(CPU_PA_RISC1_0)
+#	    if defined(CPU_PA_RISC1_0)
 		if (cpu_version == CPU_PA_RISC1_0)
 		    strcpy(un.machine, "parisc");
-	    #endif
-	    #if defined(CPU_PA_RISC1_1)
+#	    endif
+#	    if defined(CPU_PA_RISC1_1)
 		if (cpu_version == CPU_PA_RISC1_1)
 		    strcpy(un.machine, "parisc");
-	    #endif
-	    #if defined(CPU_PA_RISC1_2)
+#	    endif
+#	    if defined(CPU_PA_RISC1_2)
 		if (cpu_version == CPU_PA_RISC1_2)
 		    strcpy(un.machine, "parisc");
-	    #endif
+#	    endif
 	}
-	#endif
+#	endif
 
 	/* the uname() result goes through the arch_canon table */
 	canon = lookupInCanonTable(un.machine,
@@ -854,6 +872,17 @@ int rpmMachineScore(int type, char * name) {
 	return 0;
 }
 
+void rpmGetMachine(char **arch, char **os)
+{
+    if (arch) {
+	*arch = current[ARCH];
+    }
+
+    if (os) {
+	*os = current[OS];
+    }
+}
+
 void rpmSetMachine(char * arch, char * os) {
     int transOs = os == NULL;
     int transArch = arch == NULL;
