@@ -8,6 +8,7 @@
  */
 
 #include "config.h"
+#include "miscfn.h"
 
 #if HAVE_ASM_BYTEORDER_H
 #include <asm/byteorder.h>
@@ -16,11 +17,14 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <unistd.h>
-#include <sys/wait.h>
+#include <sys/time.h>
+#include <sys/resource.h>
 #include <sys/stat.h>
+#include <sys/wait.h>
 #include <fcntl.h>
 #include <string.h>
 
+#include "intl.h"
 #include "md5.h"
 #include "misc.h"
 #include "rpmlib.h"
@@ -92,7 +96,7 @@ int rpmReadSignature(int fd, Header *header, short sig_type)
       case RPMSIG_MD5:
       case RPMSIG_MD5_PGP:
 	rpmError(RPMERR_BADSIGTYPE,
-	      "Old (internal-only) signature!  How did you get that!?");
+	      _("Old (internal-only) signature!  How did you get that!?"));
 	return 1;
 	break;
       case RPMSIG_HEADERSIG:
@@ -215,7 +219,7 @@ static int makePGPSignature(char *file, void **sig, int_32 *size,
 	       "+batchmode=on", "+verbose=0", "+armor=off",
 	       name, "-sb", file, sigfile,
 	       NULL);
-	rpmError(RPMERR_EXEC, "Couldn't exec pgp");
+	rpmError(RPMERR_EXEC, _("Couldn't exec pgp"));
 	_exit(RPMERR_EXEC);
     }
 
@@ -226,14 +230,14 @@ static int makePGPSignature(char *file, void **sig, int_32 *size,
 
     waitpid(pid, &status, 0);
     if (!WIFEXITED(status) || WEXITSTATUS(status)) {
-	rpmError(RPMERR_SIGGEN, "pgp failed");
+	rpmError(RPMERR_SIGGEN, _("pgp failed"));
 	return 1;
     }
 
     if (stat(sigfile, &statbuf)) {
 	/* PGP failed to write signature */
 	unlink(sigfile);  /* Just in case */
-	rpmError(RPMERR_SIGGEN, "pgp failed to write signature");
+	rpmError(RPMERR_SIGGEN, _("pgp failed to write signature"));
 	return 1;
     }
 
@@ -246,7 +250,7 @@ static int makePGPSignature(char *file, void **sig, int_32 *size,
 	unlink(sigfile);
 	close(fd);
 	free(*sig);
-	rpmError(RPMERR_SIGGEN, "unable to read the signature");
+	rpmError(RPMERR_SIGGEN, _("unable to read the signature"));
 	return 1;
     }
     close(fd);
@@ -292,7 +296,8 @@ int rpmVerifySignature(char *file, int_32 sigTag, void *sig, int count,
 	    return 1;
 	}
 	break;
-      case RPMSIGTAG_LITTLEENDIANMD5:
+      case RPMSIGTAG_LEMD5_1:
+      case RPMSIGTAG_LEMD5_2:
 	if (verifyMD5Signature(file, sig, result, mdbinfileBroken)) {
 	    return 1;
 	}
@@ -387,7 +392,8 @@ static int verifyPGPSignature(char *datafile, void *sig,
 	       sigfile, datafile,
 	       NULL);
 	printf("exec failed!\n");
-	rpmError(RPMERR_EXEC, "Could not run pgp.  Use --nopgp to skip PGP checks.");
+	rpmError(RPMERR_EXEC, 
+		 _("Could not run pgp.  Use --nopgp to skip PGP checks."));
 	_exit(RPMERR_EXEC);
     }
 
@@ -421,7 +427,7 @@ char *rpmGetPassPhrase(char *prompt)
 
     if (! rpmGetVar(RPMVAR_PGP_NAME)) {
 	rpmError(RPMERR_SIGGEN,
-	      "You must set \"pgp_name:\" in your rpmrc file");
+	         _("You must set \"pgp_name:\" in your rpmrc file"));
 	return NULL;
     }
 
@@ -470,7 +476,7 @@ static int checkPassPhrase(char *passPhrase)
 	       "+batchmode=on", "+verbose=0",
 	       name, "-sf",
 	       NULL);
-	rpmError(RPMERR_EXEC, "Couldn't exec pgp");
+	rpmError(RPMERR_EXEC, _("Couldn't exec pgp"));
 	_exit(RPMERR_EXEC);
     }
 
