@@ -1,6 +1,10 @@
+#include "config.h"
+#include "miscfn.h"
+
 #include <stdlib.h>
 #include <ctype.h>
 #include <string.h>
+
 #include "stringbuf.h"
 
 #define BUF_CHUNK 1024
@@ -27,6 +31,10 @@ StringBuf newStringBuf(void)
 
 void freeStringBuf(StringBuf sb)
 {
+    if (! sb) {
+	return;
+    }
+    
     free(sb->buf);
     free(sb);
 }
