@@ -37,7 +37,7 @@ static int queryHeader(Header h, char * chptr) {
 
     str = headerSprintf(h, chptr, rpmTagTable, rpmHeaderFormats, &error);
     if (!str) {
-	fprintf(stderr, "error in format: %s\n", error);
+	fprintf(stderr, _("error in format: %s\n"), error);
 	return 1;
     }
 
@@ -442,6 +442,16 @@ int doQuery(char * prefix, enum querysources source, int queryFlags,
 	}
 	break;
 
+      case QUERY_TRIGGEREDBY:
+	if (rpmdbFindByTriggeredBy(db, arg, &matches)) {
+	    fprintf(stderr, _("no package triggers %s\n"), arg);
+	    retcode = 1;
+	} else {
+	    showMatches(db, matches, queryFlags, queryFormat);
+	    dbiFreeIndexRecord(matches);
+	}
+	break;
+
       case QUERY_WHATREQUIRES:
 	if (rpmdbFindByRequiredBy(db, arg, &matches)) {
 	    fprintf(stderr, _("no package requires %s\n"), arg);
