@@ -5,7 +5,7 @@
 
 enum querysources { QUERY_PATH, QUERY_PACKAGE, QUERY_ALL, QUERY_RPM, 
 		    QUERY_GROUP, QUERY_WHATPROVIDES, QUERY_WHATREQUIRES,
-		    QUERY_DBOFFSET };
+		    QUERY_DBOFFSET, QUERY_TRIGGEREDBY };
 
 #define QUERY_FOR_LIST		(1 << 1)
 #define QUERY_FOR_STATE		(1 << 2)
