@@ -17,6 +17,7 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 
+#include "build.h"
 #include "build/build.h"
 #include "checksig.h"
 #include "install.h"
@@ -42,6 +43,9 @@
 #define GETOPT_TIMECHECK        1012
 #define GETOPT_REBUILDDB        1013
 #define GETOPT_INSTALL		1014
+#define GETOPT_RMSOURCE		1015
+#define GETOPT_RELOCATE		1016
+#define GETOPT_TRIGGEREDBY	1017
 
 char * version = VERSION;
 
@@ -50,6 +54,126 @@ enum modes { MODE_QUERY, MODE_INSTALL, MODE_UNINSTALL, MODE_VERIFY,
 	     MODE_RECOMPILE, MODE_QUERYTAGS, MODE_INITDB, MODE_TARBUILD,
 	     MODE_REBUILDDB, MODE_UNKNOWN };
 
+/* the flags for the various options */
+static int allFiles;
+static int allMatches;
+static int badReloc;
+static int clean;
+static int dump;
+static int excldocs;
+static int force;
+static char * ftpPort;
+static char * ftpProxy;
+static int showHash;
+static int help;
+static int ignoreArch;
+static int ignoreOs;
+static int incldocs;
+static int initdb;
+static int justdb;
+static int noDeps;
+static int noOrder;
+static int noFiles;
+static int noMd5;
+static int noPgp;
+static int noScripts;
+static int oldPackage;
+static int showPercents;
+static char * pipeOutput;
+static char * prefix;
+static int queryTags;
+static int quiet;
+static int replaceFiles;
+static int replacePackages;
+static char * rootdir;
+static int shortCircuit;
+static int signIt;
+static int test;
+static int rpm_version;
+
+/* the structure describing the options we take and the defaults */
+static struct poptOption optionsTable[] = {
+	{ "addsign", '\0', 0, 0, GETOPT_ADDSIGN },
+	{ "all", 'a', 0, 0, 'a' },
+	{ "allfiles", '\0', 0, &allFiles, 0 },
+	{ "allmatches", 'a', 0, &allMatches, 0 },
+	{ "badreloc", '\0', 0, &badReloc, 0 },
+	{ "build", 'b', POPT_ARG_STRING, 0, 'b' },
+	{ "buildarch", '\0', POPT_ARG_STRING, 0, 0 },
+	{ "buildos", '\0', POPT_ARG_STRING, 0, 0 },
+	{ "buildroot", '\0', POPT_ARG_STRING, 0, GETOPT_BUILDROOT },
+	{ "checksig", 'K', 0, 0, 'K' },
+	{ "clean", '\0', 0, &clean, 0 },
+	{ "configfiles", 'c', 0, 0, 'c' },
+	{ "dbpath", '\0', POPT_ARG_STRING, 0, GETOPT_DBPATH },
+	{ "docfiles", 'd', 0, 0, 'd' },
+	{ "dump", '\0', 0, &dump, 0 },
+	{ "erase", 'e', 0, 0, 'e' },
+	{ "excludedocs", '\0', 0, &excldocs, 0},
+	{ "file", 'f', 0, 0, 'f' },
+	{ "force", '\0', 0, &force, 0 },
+	{ "ftpport", '\0', POPT_ARG_STRING, &ftpPort, 0},
+	{ "ftpproxy", '\0', POPT_ARG_STRING, &ftpProxy, 0},
+	{ "group", 'g', 0, 0, 'g' },
+	{ "hash", 'h', 0, &showHash, 0 },
+	{ "help", '\0', 0, &help, 0 },
+	{  NULL, 'i', 0, 0, 'i' },
+	{ "ignorearch", '\0', 0, &ignoreArch, 0 },
+	{ "ignoreos", '\0', 0, &ignoreOs, 0 },
+	{ "includedocs", '\0', 0, &incldocs, 0},
+	{ "initdb", '\0', 0, &initdb, 0 },
+/* info and install both using 'i' is dumb */
+	{ "install", '\0', 0, 0, GETOPT_INSTALL },
+	{ "justdb", '\0', 0, &justdb, 0 },
+	{ "list", 'l', 0, 0, 'l' },
+	{ "nodeps", '\0', 0, &noDeps, 0 },
+	{ "noorder", '\0', 0, &noOrder, 0 },
+	{ "nofiles", '\0', 0, &noFiles, 0 },
+	{ "nomd5", '\0', 0, &noMd5, 0 },
+	{ "nopgp", '\0', 0, &noPgp, 0 },
+	{ "noscripts", '\0', 0, &noScripts, 0 },
+	{ "oldpackage", '\0', 0, &oldPackage, 0 },
+	{ "package", 'p', 0, 0, 'p' },
+	{ "percent", '\0', 0, &showPercents, 0 },
+	{ "pipe", '\0', POPT_ARG_STRING, &pipeOutput, 0 },
+	{ "prefix", '\0', POPT_ARG_STRING, &prefix, 0 },
+	{ "qf", '\0', POPT_ARG_STRING, 0, GETOPT_QUERYFORMAT },
+	{ "query", 'q', 0, 0, 'q' },
+	{ "querybynumber", '\0', 0, 0, GETOPT_QUERYBYNUMBER },
+	{ "queryformat", '\0', POPT_ARG_STRING, 0, GETOPT_QUERYFORMAT },
+	{ "querytags", '\0', 0, &queryTags, 0 },
+	{ "quiet", '\0', 0, &quiet, 0 },
+	{ "rcfile", '\0', POPT_ARG_STRING, 0, 0 },
+	{ "rebuild", '\0', 0, 0, GETOPT_REBUILD },
+	{ "rebuilddb", '\0', 0, 0, GETOPT_REBUILDDB },
+	{ "recompile", '\0', 0, 0, GETOPT_RECOMPILE },
+	{ "relocate", '\0', POPT_ARG_STRING, 0, GETOPT_RELOCATE },
+	{ "replacefiles", '\0', 0, &replaceFiles, 0 },
+	{ "replacepkgs", '\0', 0, &replacePackages, 0 },
+	{ "resign", '\0', 0, 0, GETOPT_RESIGN },
+	{ "rmsource", '\0', 0, 0, GETOPT_RMSOURCE },
+	{ "root", 'r', POPT_ARG_STRING, &rootdir, 0 },
+	{ "short-circuit", '\0', 0, &shortCircuit, 0 },
+	{ "showrc", '\0', 0, 0, 0 },
+	{ "sign", '\0', 0, &signIt, 0 },
+	{ "state", 's', 0, 0, 's' },
+	{ "tarball", 't', POPT_ARG_STRING, 0, 't' },
+	{ "test", '\0', 0, &test, 0 },
+	{ "timecheck", '\0', POPT_ARG_STRING, 0, GETOPT_TIMECHECK },
+	{ "triggeredby", '\0', 0, 0, GETOPT_TRIGGEREDBY },
+	{ "upgrade", 'U', 0, 0, 'U' },
+	{ "uninstall", 'u', 0, 0, 'u' },
+	{ "verbose", 'v', 0, 0, 'v' },
+	{ "verify", 'V', 0, 0, 'V' },
+	{  NULL, 'y', 0, 0, 'V' },
+	{ "version", '\0', 0, &rpm_version, 0 },
+	{ "whatrequires", '\0', 0, 0, GETOPT_WHATREQUIRES },
+	{ "whatprovides", '\0', 0, 0, GETOPT_WHATPROVIDES },
+	{ 0, 0, 0, 0, 0 }
+};
+
+
+
 static void argerror(char * desc);
 
 static void argerror(char * desc) {
@@ -62,15 +186,13 @@ static void printVersion(void);
 static void printBanner(void);
 static void printUsage(void);
 static void printHelpLine(char * prefix, char * help);
-static int build(char *arg, int buildAmount, char *passPhrase,
-	         char *buildRootOverride, int fromTarball);
 
 static void printVersion(void) {
     printf(_("RPM version %s\n"), version);
 }
 
 static void printBanner(void) {
-    puts(_("Copyright (C) 1997 - Red Hat Software"));
+    puts(_("Copyright (C) 1998 - Red Hat Software"));
     puts(_("This may be freely redistributed under the terms of the GNU "
 	   "Public License"));
 }
@@ -89,14 +211,16 @@ static void printUsage(void) {
     puts(_("                        [--rcfile <file>] [--ignorearch] [--dbpath <dir>]"));
     puts(_("                        [--prefix <dir>] [--ignoreos] [--nodeps] [--allfiles]"));
     puts(_("                        [--ftpproxy <host>] [--ftpport <port>] [--justdb]"));
-    puts(_("                        [--noorder] file1.rpm ... fileN.rpm"));
+    puts(_("                        [--noorder] [--relocate oldpath=newpath]"));
+    puts(_("                        [--badreloc] file1.rpm ... fileN.rpm"));
     puts(_("       rpm {--upgrade -U} [-v] [--hash -h] [--percent] [--force] [--test]"));
     puts(_("                        [--oldpackage] [--root <dir>] [--noscripts]"));
     puts(_("                        [--excludedocs] [--includedocs] [--rcfile <file>]"));
     puts(_("                        [--ignorearch]  [--dbpath <dir>] [--prefix <dir>] "));
     puts(_("                        [--ftpproxy <host>] [--ftpport <port>]"));
     puts(_("                        [--ignoreos] [--nodeps] [--allfiles] [--justdb]"));
-    puts(_("                        [--noorder] file1.rpm ... fileN.rpm"));
+    puts(_("                        [--noorder] [--relocate oldpath=newpath]"));
+    puts(_("                        [--badreloc] file1.rpm ... fileN.rpm"));
     puts(_("       rpm {--query -q} [-afpg] [-i] [-l] [-s] [-d] [-c] [-v] [-R]"));
     puts(_("                        [--scripts] [--root <dir>] [--rcfile <file>]"));
     puts(_("                        [--whatprovides] [--whatrequires] [--requires]"));
@@ -113,7 +237,8 @@ static void printUsage(void) {
     puts(_("                        [--justdb] package1 ... packageN"));
     puts(_("       rpm {-b|t}[plciba] [-v] [--short-circuit] [--clean] [--rcfile  <file>]"));
     puts(_("                        [--sign] [--test] [--timecheck <s>] [--buildos <os>]"));
-    puts(_("                        [--buildarch <arch>] specfile"));
+    puts(_("                        [--buildarch <arch>] [--rmsource] specfile"));
+    puts(_("       rpm {--rmsource} [--rcfile <file>] [-v] specfile"));
     puts(_("       rpm {--rebuild} [--rcfile <file>] [-v] source1.rpm ... sourceN.rpm"));
     puts(_("       rpm {--recompile} [--rcfile <file>] [-v] source1.rpm ... sourceN.rpm"));
     puts(_("       rpm {--resign} [--rcfile <file>] package1 package2 ... packageN"));
@@ -243,6 +368,10 @@ static void printHelp(void) {
     puts(         "    --install <packagefile>");
     printHelpLine("    -i <packagefile>      ",
 		  _("install package"));
+    printHelpLine("      --relocate <oldpath>=<newpath>",
+		  _("relocate files from <oldpath> to <newpath>"));
+    printHelpLine("      --badreloc",
+		  _("relocate files even though the package doesn't allow it"));
     printHelpLine("      --prefix <dir>      ",
 		  _("relocate the package to <dir>, if relocatable"));
     printHelpLine("      --dbpath <dir>      ",
@@ -325,6 +454,8 @@ static void printHelp(void) {
 		  _("skip straight to specified stage (only for c,i)"));
     printHelpLine("      --clean             ",
 		  _("remove build tree when done"));
+    printHelpLine("      --rmsource          ",
+		  _("remove sources and spec file when done"));
     printHelpLine("      --sign              ",
 		  _("generate PGP signature"));
     printHelpLine("      --buildroot <dir>     ",
@@ -340,6 +471,8 @@ static void printHelp(void) {
     puts("");
     printHelpLine("    --rebuild <src_pkg>   ",
 		  _("install source package, build binary package and remove spec file, sources, patches, and icons."));
+    printHelpLine("    --rmsource <spec>   ",
+		  _("remove sources and spec file"));
     printHelpLine("    --recompile <src_pkg> ",
 		  _("like --rebuild, but don't build any package"));
     printHelpLine("    --resign <pkg>+       ",
@@ -365,172 +498,28 @@ static void printHelp(void) {
 		  _("use <dir> as the top level directory"));
 }
 
-static int build(char *arg, int buildAmount, char *passPhrase,
-	         char *buildRootOverride, int fromTarball) {
-    FILE *f;
-    Spec *s, *specArray;
-    char * specfile;
-    int res = 0;
-    struct stat statbuf;
-    char * specDir;
-    char * tmpSpecFile;
-    char * cmd;
-    char buf[1024];
-    int flags;
-
-    if (fromTarball) {
-	specDir = rpmGetVar(RPMVAR_SPECDIR);
-	tmpSpecFile = alloca(1024);
-	sprintf(tmpSpecFile, "%s/rpm-spec-file-%d", specDir, (int) getpid());
-
-	cmd = alloca(strlen(arg) + 50 + strlen(tmpSpecFile));
-	sprintf(cmd, "gunzip < %s | tar xOvf - \\*.spec 2>&1 > %s", arg,
-			tmpSpecFile);
-	if (!(f = popen(cmd, "r"))) {
-	    fprintf(stderr, _("Failed to open tar pipe: %s\n"), 
-			strerror(errno));
-	    return 1;
-	}
-	if (!fgets(buf, sizeof(buf) - 1, f)) {
-	    fprintf(stderr, _("Failed to read spec file from %s\n"), arg);
-	    unlink(tmpSpecFile);
-	    return 1;
-	}
-	pclose(f);
-
-	cmd = specfile = buf;
-	while (*cmd) {
-	    if (*cmd == '/') specfile = cmd + 1;
-	    cmd++;
-	}
-
-	cmd = specfile;
-
-	/* remove trailing \n */
-	specfile = cmd + strlen(cmd) - 1;
-	*specfile = '\0';
-
-	specfile = alloca(strlen(specDir) + strlen(cmd) + 5);
-	sprintf(specfile, "%s/%s", specDir, cmd);
-	
-	if (rename(tmpSpecFile, specfile)) {
-	    fprintf(stderr, _("Failed to rename %s to %s: %s\n"),
-		    tmpSpecFile, specfile, strerror(errno));
-	    unlink(tmpSpecFile);
-	    return 1;
-	}
-
-	/* Make the directory which contains the tarball the source 
-	   directory for this run */
-
-	if (*arg != '/') {
-	    /* XXX this is broken if PWD is near 1024 */
-	    getcwd(buf, 1024);
-	    strcat(buf, "/");
-	    strcat(buf, arg);
-	} else 
-	    strcpy(buf, arg);
-
-	cmd = buf + strlen(buf) - 1;
-	while (*cmd != '/') cmd--;
-	*cmd = '\0';
-
-	rpmSetVar(RPMVAR_SOURCEDIR, buf);
-    } else if (arg[0] == '/') {
-	specfile = arg;
-    } else {
-	/* XXX this is broken if PWD is near 1024 */
-	specfile = alloca(1024);
-	getcwd(specfile, 1024);
-	strcat(specfile, "/");
-	strcat(specfile, arg);
-    }
-
-    stat(specfile, &statbuf);
-    if (! S_ISREG(statbuf.st_mode)) {
-	rpmError(RPMERR_BADSPEC, _("File is not a regular file: %s\n"),
-		 specfile);
-	return 1;
-    }
-    
-    if (!(f = fopen(specfile, "r"))) {
-	fprintf(stderr, _("unable to open: %s\n"), specfile);
-	return 1;
-    }
-    
-    s = specArray = parseSpec(f, specfile, buildRootOverride);
-    fclose(f);
-    if (! specArray) {
-	/* Spec parse failed -- could be Exclude: Exclusive: */
-	res = 1;
-	if (rpmErrorCode() == RPMERR_BADARCH) {
-	    fprintf(stderr, _("%s doesn't build on this architecture\n"), arg);
-	} else {
-	    fprintf(stderr, _("Build failed.\n"));
-	}
-    } else {
-	while (*s && !res) {
-	    if (verifySpec(*s)) {
-		fprintf(stderr, _("\n%cSpec file check failed!!\n"), 7);
-		fprintf(stderr,
-			_("Tell rpm-list@redhat.com if this is incorrect.\n\n"));
-		res = 1;
-	    } else {
-		flags = buildAmount;
-		/* Don't build source package or remove sources */
-		/* unless this is the last package being built. */
-		if (*(s+1)) {
-		    flags = flags & ~RPMBUILD_SOURCE;
-		    flags = flags & ~RPMBUILD_RMSOURCE;
-		}
-		if (doBuild(*s, flags, passPhrase)) {
-		    fprintf(stderr, _("Build failed.\n"));
-		    res = 1;
-		}
-	    }
-	    s++;
-	}
-
-	s = specArray;
-	while (*s) {
-	    freeSpec(*s++);
-	}
-	free(specArray);
-    }
-
-    if (fromTarball) unlink(specfile);
-
-    return res;
-}
-
 int main(int argc, char ** argv) {
     enum modes bigMode = MODE_UNKNOWN;
     enum querysources querySource = QUERY_PACKAGE;
     enum verifysources verifySource = VERIFY_PACKAGE;
     int arg, len;
-    int queryFor = 0, test = 0, version = 0, help = 0, force = 0;
-    int quiet = 0, replaceFiles = 0, replacePackages = 0, showPercents = 0;
-    int showHash = 0, installFlags = 0, uninstallFlags = 0, interfaceFlags = 0;
-    int buildAmount = 0, oldPackage = 0, clean = 0, signIt = 0;
-    int shortCircuit = 0, queryTags = 0, excldocs = 0;
-    int incldocs = 0, noScripts = 0, noDeps = 0, allMatches = 0, noOrder = 0;
-    int noPgp = 0, dump = 0, initdb = 0, ignoreArch = 0, showrc = 0;
-    int gotDbpath = 0, building = 0, ignoreOs = 0, noFiles = 0, verifyFlags;
-    int noMd5 = 0, allFiles = 0, justdb = 0;
+    int queryFor = 0;
+    int installFlags = 0, uninstallFlags = 0, interfaceFlags = 0;
+    int buildAmount = 0;
+    int showrc = 0;
+    int gotDbpath = 0, building = 0, verifyFlags;
+    int rmsource = 0;
     int checksigFlags = 0;
-    char *tce;
     int timeCheck = 0;
     int addSign = NEW_SIGNATURE;
-    char * rcfile = NULL, * queryFormat = NULL, * prefix = NULL;
+    char * rcfile = NULL, * queryFormat = NULL;
     char buildChar = ' ';
-    char * rootdir = "/";
-    char * pipeOutput = NULL;
     char * specFile;
-    char *passPhrase = "";
-    char *buildRootOverride = NULL;
-    char *arch = NULL;
-    char * ftpProxy = NULL, * ftpPort = NULL;
-    char *os = NULL;
+    char * tce;
+    char * passPhrase = "";
+    char * buildRootOverride = NULL, * cookie = NULL;
+    char * arch = NULL;
+    char * os = NULL;
     char * optArg;
     pid_t pipeChild = 0;
     char * pkg;
@@ -542,81 +531,45 @@ int main(int argc, char ** argv) {
     int ec = 0;
     int status;
     int p[2];
-    struct poptOption optionsTable[] = {
-	    { "addsign", '\0', 0, 0, GETOPT_ADDSIGN },
-	    { "all", 'a', 0, 0, 'a' },
-	    { "allfiles", '\0', 0, &allFiles, 0 },
-	    { "allmatches", 'a', 0, &allMatches, 0 },
-	    { "build", 'b', POPT_ARG_STRING, 0, 'b' },
-	    { "buildarch", '\0', POPT_ARG_STRING, 0, 0 },
-	    { "buildos", '\0', POPT_ARG_STRING, 0, 0 },
-	    { "buildroot", '\0', POPT_ARG_STRING, 0, GETOPT_BUILDROOT },
-	    { "checksig", 'K', 0, 0, 'K' },
-	    { "clean", '\0', 0, &clean, 0 },
-	    { "configfiles", 'c', 0, 0, 'c' },
-	    { "dbpath", '\0', POPT_ARG_STRING, 0, GETOPT_DBPATH },
-	    { "docfiles", 'd', 0, 0, 'd' },
-	    { "dump", '\0', 0, &dump, 0 },
-	    { "erase", 'e', 0, 0, 'e' },
-            { "excludedocs", '\0', 0, &excldocs, 0},
-	    { "file", 'f', 0, 0, 'f' },
-	    { "force", '\0', 0, &force, 0 },
-	    { "ftpport", '\0', POPT_ARG_STRING, &ftpPort, 0},
-	    { "ftpproxy", '\0', POPT_ARG_STRING, &ftpProxy, 0},
-	    { "group", 'g', 0, 0, 'g' },
-	    { "hash", 'h', 0, &showHash, 0 },
-	    { "help", '\0', 0, &help, 0 },
-	    {  NULL, 'i', 0, 0, 'i' },
-	    { "ignorearch", '\0', 0, &ignoreArch, 0 },
-	    { "ignoreos", '\0', 0, &ignoreOs, 0 },
-            { "includedocs", '\0', 0, &incldocs, 0},
-	    { "initdb", '\0', 0, &initdb, 0 },
-	/* info and install both using 'i' is dumb */
-	    { "install", '\0', 0, 0, GETOPT_INSTALL },
-	    { "justdb", '\0', 0, &justdb, 0 },
-	    { "list", 'l', 0, 0, 'l' },
-	    { "nodeps", '\0', 0, &noDeps, 0 },
-	    { "noorder", '\0', 0, &noOrder, 0 },
-	    { "nofiles", '\0', 0, &noFiles, 0 },
-	    { "nomd5", '\0', 0, &noMd5, 0 },
-	    { "nopgp", '\0', 0, &noPgp, 0 },
-	    { "noscripts", '\0', 0, &noScripts, 0 },
-	    { "oldpackage", '\0', 0, &oldPackage, 0 },
-	    { "package", 'p', 0, 0, 'p' },
-	    { "percent", '\0', 0, &showPercents, 0 },
-	    { "pipe", '\0', POPT_ARG_STRING, &pipeOutput, 0 },
-	    { "prefix", '\0', POPT_ARG_STRING, &prefix, 0 },
-	    { "qf", '\0', POPT_ARG_STRING, 0, GETOPT_QUERYFORMAT },
-	    { "query", 'q', 0, 0, 'q' },
-	    { "querybynumber", '\0', 0, 0, GETOPT_QUERYBYNUMBER },
-	    { "queryformat", '\0', POPT_ARG_STRING, 0, GETOPT_QUERYFORMAT },
-	    { "querytags", '\0', 0, &queryTags, 0 },
-	    { "quiet", '\0', 0, &quiet, 0 },
-	    { "rcfile", '\0', POPT_ARG_STRING, 0, 0 },
-	    { "recompile", '\0', 0, 0, GETOPT_RECOMPILE },
-	    { "rebuild", '\0', 0, 0, GETOPT_REBUILD },
-	    { "rebuilddb", '\0', 0, 0, GETOPT_REBUILDDB },
-	    { "replacefiles", '\0', 0, &replaceFiles, 0 },
-	    { "replacepkgs", '\0', 0, &replacePackages, 0 },
-	    { "resign", '\0', 0, 0, GETOPT_RESIGN },
-	    { "root", 'r', POPT_ARG_STRING, &rootdir, 0 },
-	    { "short-circuit", '\0', 0, &shortCircuit, 0 },
-	    { "showrc", '\0', 0, 0, 0 },
-	    { "sign", '\0', 0, &signIt, 0 },
-	    { "state", 's', 0, 0, 's' },
-	    { "tarball", 't', POPT_ARG_STRING, 0, 't' },
-	    { "test", '\0', 0, &test, 0 },
-	    { "timecheck", '\0', POPT_ARG_STRING, 0, GETOPT_TIMECHECK },
-	    { "upgrade", 'U', 0, 0, 'U' },
-	    { "uninstall", 'u', 0, 0, 'u' },
-	    { "verbose", 'v', 0, 0, 'v' },
-	    { "verify", 'V', 0, 0, 'V' },
-	    {  NULL, 'y', 0, 0, 'V' },
-	    { "version", '\0', 0, &version, 0 },
-	    { "whatrequires", '\0', 0, 0, GETOPT_WHATREQUIRES },
-	    { "whatprovides", '\0', 0, 0, GETOPT_WHATPROVIDES },
-	    { 0, 0, 0, 0, 0 } 
-	} ;
+    struct rpmRelocation * relocations = NULL;
+    int numRelocations = 0;
+	
+    /* set the defaults for the various command line options */
+    allFiles = 0;
+    allMatches = 0;
+    badReloc = 0;
+    clean = 0;
+    dump = 0;
+    excldocs = 0;
+    force = 0;
+    ftpProxy = NULL;
+    ftpPort = NULL;
+    showHash = 0;
+    help = 0;
+    ignoreArch = 0;
+    ignoreOs = 0;
+    incldocs = 0;
+    initdb = 0;
+    justdb = 0;
+    noDeps = 0;
+    noOrder = 0;
+    noFiles = 0;
+    noMd5 = 0;
+    noPgp = 0;
+    noScripts = 0;
+    oldPackage = 0;
+    showPercents = 0;
+    pipeOutput = NULL;
+    prefix = NULL;
+    queryTags = 0;
+    quiet = 0;
+    replaceFiles = 0;
+    replacePackages = 0;
+    rootdir = "/";
+    shortCircuit = 0;
+    signIt = 0;
+    test = 0;
+    rpm_version = 0;
 
     /* set up the correct locale */
     setlocale(LC_ALL, "" );
@@ -643,6 +596,7 @@ int main(int argc, char ** argv) {
 		   !strcmp(*currarg, "--tarbuild") ||
 		   !strcmp(*currarg, "--build") ||
 		   !strcmp(*currarg, "--rebuild") ||
+		   !strcmp(*currarg, "--rmsource") ||
 		   !strcmp(*currarg, "--recompile")) {
 	    building = 1;
 	}
@@ -833,12 +787,27 @@ int main(int argc, char ** argv) {
 	    querySource = QUERY_WHATPROVIDES;
 	    break;
 
+	  case GETOPT_TRIGGEREDBY:
+	    if (querySource != QUERY_PACKAGE && 
+		querySource != QUERY_TRIGGEREDBY)
+		argerror(_("one type of query/verify may be performed at a "
+				"time"));
+	    querySource = QUERY_TRIGGEREDBY;
+	    break;
+
 	  case GETOPT_REBUILD:
 	    if (bigMode != MODE_UNKNOWN && bigMode != MODE_REBUILD)
 		argerror(_("only one major mode may be specified"));
 	    bigMode = MODE_REBUILD;
 	    break;
 
+	  case GETOPT_RMSOURCE:
+	    if (bigMode != MODE_UNKNOWN && bigMode != MODE_BUILD &&
+		bigMode != MODE_TARBUILD)
+		argerror(_("only one major mode may be specified"));
+	    rmsource = 1;
+	    break;
+
 	  case GETOPT_RECOMPILE:
 	    if (bigMode != MODE_UNKNOWN && bigMode != MODE_RECOMPILE)
 		argerror(_("only one major mode may be specified"));
@@ -890,6 +859,7 @@ int main(int argc, char ** argv) {
 		argerror("Argument to --timecheck must be integer");
 	    }
 	    rpmSetVar(RPMVAR_TIMECHECK, optArg);
+	    timeCheck = 1;
 	    break;
 
 	  case GETOPT_REBUILDDB:
@@ -898,6 +868,20 @@ int main(int argc, char ** argv) {
 	    bigMode = MODE_REBUILDDB;
 	    break;
 
+	  case GETOPT_RELOCATE:
+	    if (*optArg != '/') 
+		argerror(_("relocations must begin with a /"));
+	    if (!(errString = strchr(optArg, '=')))
+		argerror(_("relocations must contain a ="));
+	    *errString++ = '\0';
+	    if (*errString != '/') 
+		argerror(_("relocations must have a / following the ="));
+	    relocations = realloc(relocations, 
+				  sizeof(*relocations) * (numRelocations + 1));
+	    relocations[numRelocations].oldPath = optArg;
+	    relocations[numRelocations++].newPath = errString;
+	    break;
+
 	  default:
 	    fprintf(stderr, "Internal error in argument processing :-(\n");
 	    exit(1);
@@ -907,7 +891,7 @@ int main(int argc, char ** argv) {
     if (quiet)
 	rpmSetVerbosity(RPMMESS_QUIET);
 
-    if (version) printVersion();
+    if (rpm_version) printVersion();
     if (help) printHelp();
 
     if (arg < -1) {
@@ -917,6 +901,9 @@ int main(int argc, char ** argv) {
 	exit(1);
     }
 
+    if (rmsource && bigMode == MODE_UNKNOWN)
+	bigMode = MODE_BUILD;
+    
     if (initdb)
 	if (bigMode != MODE_UNKNOWN) 
 	    argerror(_("only one major mode may be specified"));
@@ -957,6 +944,15 @@ int main(int argc, char ** argv) {
     if (bigMode != MODE_INSTALL && force)
 	argerror(_("only installation and upgrading may be forced"));
 
+    if (bigMode != MODE_INSTALL && badReloc)
+	argerror(_("files may only be relocated during package installation"));
+
+    if (relocations && prefix)
+	argerror(_("only one of --prefix or --relocate may be used"));
+
+    if (bigMode != MODE_INSTALL && relocations)
+	argerror(_("--relocate may only be used when installing new packages"));
+
     if (bigMode != MODE_INSTALL && prefix)
 	argerror(_("--prefix may only be used when installing new packages"));
 
@@ -1039,8 +1035,11 @@ int main(int argc, char ** argv) {
     if (rootdir && rootdir[0] != '/')
 	argerror(_("arguments to --root (-r) must begin with a /"));
 
-    if (bigMode != MODE_BUILD && bigMode != MODE_TARBUILD && clean) 
-	argerror(_("--clean may only be used during package building"));
+    if (bigMode != MODE_BUILD && bigMode != MODE_TARBUILD && clean)
+	argerror(_("--clean may only be used with -b and -t"));
+
+    if (bigMode != MODE_BUILD && bigMode != MODE_TARBUILD && rmsource)
+	argerror(_("--rmsource may only be used with -b and -t"));
 
     if (bigMode != MODE_BUILD && bigMode != MODE_TARBUILD && shortCircuit) 
 	argerror(_("--short-circuit may only be used during package building"));
@@ -1125,7 +1124,7 @@ int main(int argc, char ** argv) {
 	
     switch (bigMode) {
       case MODE_UNKNOWN:
-	if (!version && !help) printUsage();
+	if (!rpm_version && !help) printUsage();
 	break;
 
       case MODE_REBUILDDB:
@@ -1163,20 +1162,24 @@ int main(int argc, char ** argv) {
 	if (!poptPeekArg(optCon))
 	    argerror(_("no packages files given for rebuild"));
 
-	buildAmount = RPMBUILD_PREP | RPMBUILD_BUILD | RPMBUILD_INSTALL |
-	    RPMBUILD_SWEEP | RPMBUILD_RMSOURCE;
+	buildAmount = RPMBUILD_PREP | RPMBUILD_BUILD | RPMBUILD_INSTALL;
 	if (bigMode == MODE_REBUILD) {
-	    buildAmount |= RPMBUILD_BINARY;
+	    buildAmount |= RPMBUILD_PACKAGEBINARY;
+	    buildAmount |= RPMBUILD_RMSOURCE;
+	    buildAmount |= RPMBUILD_CLEAN;
+	    buildAmount |= RPMBUILD_RMBUILD;
 	}
 
 	while ((pkg = poptGetArg(optCon))) {
-	    if (doSourceInstall("/", pkg, &specFile, NULL))
+	    if (doSourceInstall("/", pkg, &specFile, &cookie))
 		exit(1);
 
 	    if (build(specFile, buildAmount, passPhrase, buildRootOverride,
-			0)) {
+			0, test, cookie)) {
 		exit(1);
 	    }
+	    free(cookie);
+	    free(specFile);
 	}
 	break;
 
@@ -1188,9 +1191,10 @@ int main(int argc, char ** argv) {
 	switch (buildChar) {
 	  /* these fallthroughs are intentional */
 	  case 'a':
-	    buildAmount |= RPMBUILD_SOURCE;
+	    buildAmount |= RPMBUILD_PACKAGESOURCE;
 	  case 'b':
-	    buildAmount |= RPMBUILD_BINARY;
+	    buildAmount |= RPMBUILD_PACKAGEBINARY;
+	    buildAmount |= RPMBUILD_CLEAN;
 	  case 'i':
 	    buildAmount |= RPMBUILD_INSTALL;
 	    if ((buildChar == 'i') && shortCircuit)
@@ -1204,25 +1208,26 @@ int main(int argc, char ** argv) {
 	    break;
 	    
 	  case 'l':
-	    buildAmount |= RPMBUILD_LIST;
+	    buildAmount |= RPMBUILD_FILECHECK;
 	    break;
 	}
 
+	if (rmsource)
+	    buildAmount |= RPMBUILD_RMSOURCE;
+
 	if (clean)
-	    buildAmount |= RPMBUILD_SWEEP;
+	    buildAmount |= RPMBUILD_RMBUILD;
 
-	if (test)
-	    buildAmount |= RPMBUILD_TEST;
-
-	if (!poptPeekArg(optCon))
+	if (!poptPeekArg(optCon)) {
 	    if (bigMode == MODE_BUILD)
 		argerror(_("no spec files given for build"));
 	    else
 		argerror(_("no tar files given for build"));
+	}
 
 	while ((pkg = poptGetArg(optCon)))
 	    if (build(pkg, buildAmount, passPhrase, buildRootOverride,
-			bigMode == MODE_TARBUILD)) {
+			bigMode == MODE_TARBUILD, test, NULL)) {
 		exit(1);
 	    }
 	break;
@@ -1245,6 +1250,7 @@ int main(int argc, char ** argv) {
 	if (force)
 	    installFlags |= (RPMINSTALL_REPLACEPKG | RPMINSTALL_REPLACEFILES);
 	if (replaceFiles) installFlags |= RPMINSTALL_REPLACEFILES;
+	if (badReloc) installFlags |= RPMINSTALL_FORCERELOCATE;
 	if (replacePackages) installFlags |= RPMINSTALL_REPLACEPKG;
 	if (test) installFlags |= RPMINSTALL_TEST;
 	if (noScripts) installFlags |= RPMINSTALL_NOSCRIPTS;
@@ -1268,8 +1274,21 @@ int main(int argc, char ** argv) {
 	if (!poptPeekArg(optCon))
 	    argerror(_("no packages given for install"));
 
-	ec += doInstall(rootdir, poptGetArgs(optCon), prefix, installFlags, 
-			interfaceFlags);
+	/* we've already ensured !(!prefix && !relocations) */
+	if (prefix) {
+	    relocations = alloca(2 * sizeof(*relocations));
+	    relocations[0].oldPath = NULL;   /* special case magic */
+	    relocations[0].newPath = prefix;
+	    relocations[1].oldPath = relocations[1].newPath = NULL;
+	} else if (relocations) {
+	    relocations = realloc(relocations, 
+				  sizeof(*relocations) * (numRelocations + 1));
+	    relocations[numRelocations].oldPath = NULL;
+	    relocations[numRelocations].newPath = NULL;
+	}
+
+	ec += doInstall(rootdir, poptGetArgs(optCon), installFlags, 
+			interfaceFlags, relocations);
 	break;
 
       case MODE_QUERY:
