@@ -6,8 +6,9 @@
 
 #include "header.h"
 #include "rpmlib.h"
+#include "intl.h"
 
-void main(int argc, char ** argv)
+int main(int argc, char ** argv)
 {
     Header h;
     int fd;
@@ -32,6 +33,8 @@ void main(int argc, char ** argv)
   
     headerDump(h, stdout, 1, rpmTagTable);
     headerFree(h);
+
+    return 0;
 }
 
   
